# Name:         mapper_opendap_sentinel2.py
# Purpose:      Nansat mapping for ESA Sentinel-2 data from the Norwegian ground segment
# Author:       Morten W. Hansen
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html
import os
from datetime import datetime
import json
import warnings

import numpy as np
from netCDF4 import Dataset
import gdal

import pythesint as pti

from nansat.mappers.opendap import Opendap
from nansat.nsr import NSR


class Mapper(Opendap):

    baseURLs = [
            'http://nbstds.met.no/thredds/dodsC/NBS/S2A',
            'http://nbstds.met.no/thredds/dodsC/NBS/S2B',
    ]

    timeVarName = 'time'
    xName = 'x'
    yName = 'y'
    timeCalendarStart = '1981-01-01'
    GCP_STEP=100

    def __init__(self, filename, gdal_dataset, gdal_metadata, date=None,
                 ds=None, bands=None, cachedir=None, *args, **kwargs):

        self.test_mapper(filename)
        timestamp = date if date else self.get_date(filename)

        self.create_vrt(filename, gdal_dataset, gdal_metadata, timestamp, ds, bands, cachedir)

        mditem = 'entry_title'
        if not self.dataset.GetMetadataItem(mditem):
            try:
                self.dataset.SetMetadataItem(mditem, str(self.ds.getncattr('title')))
            except AttributeError:
                self.dataset.SetMetadataItem(mditem, filename)
        mditem = 'data_center'
        if not self.dataset.GetMetadataItem(mditem):
            self.dataset.SetMetadataItem('data_center', json.dumps(pti.get_gcmd_provider('NO/MET')))
        mditem = 'ISO_topic_category'
        if not self.dataset.GetMetadataItem(mditem):
            self.dataset.SetMetadataItem(mditem,
                pti.get_iso19115_topic_category('Imagery/Base Maps/Earth Cover')['iso_topic_category'])

        mm = pti.get_gcmd_instrument('multi-spectral')
        ee = pti.get_gcmd_platform('sentinel-2')
        self.dataset.SetMetadataItem('instrument', json.dumps(mm))
        self.dataset.SetMetadataItem('platform', json.dumps(ee))

    @staticmethod
    def get_date(filename):
        """Extract date and time parameters from filename and return
        it as a formatted (isoformat) string

        Parameters
        ----------

        filename: str
            nn

        Returns
        -------
            str, YYYY-mm-ddThh:MMZ

        """
        _, filename = os.path.split(filename)
        t = datetime.strptime(filename.split('_')[2], '%Y%m%dT%H%M%S')
        return datetime.strftime(t, '%Y-%m-%dT%H:%M:%SZ')

    def convert_dstime_datetimes(self, ds_time):
        """Convert time variable to np.datetime64"""
        ds_datetimes = np.array(
            [(np.datetime64(self.timeCalendarStart).astype('M8[s]')
              + np.timedelta64(int(sec), 's').astype('m8[s]')) for sec in ds_time]).astype('M8[s]')
        return ds_datetimes




# ------------------------------------------------------------------------------
# Name:         test_vrt.py
# Purpose:      Test the VRT class
#
# Author:       Anton Korosov
#
# Created:      15.01.2018
# Copyright:    (c) NERSC
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html
# ------------------------------------------------------------------------------
from __future__ import absolute_import, unicode_literals
import unittest
import logging
import os
from mock import patch, PropertyMock, Mock, MagicMock, DEFAULT

import xml.etree.ElementTree as ET
import warnings

import numpy as np
import gdal
import pythesint as pti

from nansat.node import Node
from nansat.nsr import NSR
from nansat.vrt import VRT
from nansat.tests.nansat_test_base import NansatTestBase

from nansat.exceptions import NansatProjectionError


class VRTTest(NansatTestBase):
    nsr_wkt = ('GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",'
                '6378137,298.257223563,AUTHORITY["EPSG","7030"]],AUT'
                'HORITY["EPSG","6326"]],PRIMEM["Greenwich",0,AUTHORI'
                'TY["EPSG","8901"]],UNIT["degree",0.0174532925199433'
                ',AUTHORITY["EPSG","9122"]],AUTHORITY["EPSG","4326"]]')

    @patch.object(VRT, '_make_filename', return_value='/vsimem/filename.vrt')
    def test_init(self, mock_make_filename):
        metadata={'key': 'value'}
        vrt = VRT(metadata=metadata)

        self.assertIsInstance(vrt, VRT)
        self.assertEqual(vrt.filename, '/vsimem/filename.vrt')
        self.assertIsInstance(vrt.dataset, gdal.Dataset)
        self.assertIsInstance(vrt.logger, logging.Logger) # just for testing mocking
        self.assertIsInstance(vrt.driver, gdal.Driver)
        self.assertEqual(vrt.band_vrts, {})
        self.assertEqual(vrt.tps, False)
        self.assertTrue(vrt.vrt is None)
        self.assertTrue(vrt.xml.startswith('<VRTDataset rasterXSize="1" rasterYSize="1"'))
        self.assertTrue(mock_make_filename.called_once())
        self.assertEqual(vrt.dataset.GetMetadata(), metadata)

    @patch.object(VRT, '_make_filename', return_value='filename.vrt')
    def test_del(self, mock_make_filename):
        vrt = VRT()
        vrt = None
        self.assertFalse(os.path.exists('filename.vrt'))

    @patch.object(VRT, '_init_from_gdal_dataset')
    def test_from_gdal_dataset(self, _init_from_gdal_dataset):
        ds = gdal.Open(self.test_file_gcps)
        vrt = VRT.from_gdal_dataset(ds)
        self.assertIsInstance(vrt, VRT)
        self.assertTrue(_init_from_gdal_dataset.called_once())

    @patch.object(VRT, '_add_geolocation')
    def test_init_from_gdal_dataset(self, _add_geolocation):
        vrt = VRT()
        ds = gdal.Open(self.test_file_gcps)
        vrt._init_from_gdal_dataset(ds)

        self.assertEqual(vrt.dataset.RasterXSize, ds.RasterXSize)
        self.assertEqual(vrt.dataset.RasterYSize, ds.RasterYSize)
        self.assertEqual(vrt.dataset.GetProjection(), ds.GetProjection())
        self.assertEqual(vrt.dataset.GetGeoTransform(), ds.GetGeoTransform())
        self.assertEqual(vrt.dataset.GetGCPProjection(), ds.GetGCPProjection())
        self.assertIn('filename', list(vrt.dataset.GetMetadata().keys()))
        self.assertTrue(_add_geolocation.called_once())

    def test_from_dataset_params(self):
        ds = gdal.Open(self.test_file_gcps)
        vrt = VRT.from_dataset_params(ds.RasterXSize,
                                      ds.RasterYSize,
                                      ds.GetGeoTransform(),
                                      ds.GetProjection(),
                                      ds.GetGCPs(),
                                      ds.GetGCPProjection())

        self.assertEqual(vrt.dataset.RasterXSize, ds.RasterXSize)
        self.assertEqual(vrt.dataset.RasterYSize, ds.RasterYSize)
        self.assertEqual(vrt.dataset.GetProjection(), ds.GetProjection())
        self.assertEqual(vrt.dataset.GetGeoTransform(), ds.GetGeoTransform())
        self.assertEqual(vrt.dataset.GetGCPProjection(), ds.GetGCPProjection())
        self.assertIn('filename', list(vrt.dataset.GetMetadata().keys()))

    def test_from_array(self):
        array = gdal.Open(self.test_file_gcps).ReadAsArray()[1, 10:, :]
        vrt = VRT.from_array(array)

        self.assertEqual(vrt.dataset.RasterXSize, array.shape[1])
        self.assertEqual(vrt.dataset.RasterYSize, array.shape[0])
        self.assertEqual(vrt.dataset.RasterCount, 1)
        self.assertIn('filename', list(vrt.dataset.GetMetadata().keys()))
        self.assertEqual(gdal.Unlink(vrt.filename.replace('.vrt', '.raw')), 0)

    def test_from_lonlat(self):
        geo_keys = ['LINE_OFFSET', 'LINE_STEP', 'PIXEL_OFFSET', 'PIXEL_STEP', 'SRS',
                    'X_BAND', 'X_DATASET', 'Y_BAND', 'Y_DATASET']
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt = VRT.from_lonlat(lon, lat, n_gcps=25)

        self.assertEqual(vrt.dataset.RasterXSize, 10)
        self.assertEqual(vrt.dataset.RasterYSize, 30)
        self.assertIn('filename', list(vrt.dataset.GetMetadata().keys()))
        geo_metadata = vrt.dataset.GetMetadata(str('GEOLOCATION'))
        for geo_key in geo_keys:
            self.assertEqual(vrt.geolocation.data[geo_key], geo_metadata[geo_key])
        self.assertIsInstance(vrt.geolocation.x_vrt, VRT)
        self.assertIsInstance(vrt.geolocation.y_vrt, VRT)
        self.assertEqual(vrt.geolocation.x_vrt.filename, geo_metadata['X_DATASET'])
        self.assertEqual(vrt.geolocation.y_vrt.filename, geo_metadata['Y_DATASET'])
        self.assertEqual(len(vrt.dataset.GetGCPs()), 25)

    def test_from_lonlat_no_gcps(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt = VRT.from_lonlat(lon, lat, add_gcps=False)
        self.assertEqual(len(vrt.dataset.GetGCPs()), 0)

    def test_copy_empty_vrt(self):
        vrt1 = VRT()
        vrt2 = vrt1.copy()

        self.assertIsInstance(vrt2, VRT)
        self.assertIsInstance(vrt2.filename, str)
        self.assertEqual(vrt2.dataset.RasterXSize, vrt1.dataset.RasterXSize)
        self.assertEqual(vrt2.dataset.RasterYSize, vrt1.dataset.RasterYSize)
        self.assertEqual(vrt2.dataset.GetProjection(), vrt1.dataset.GetProjection())
        self.assertEqual(vrt2.dataset.GetGeoTransform(), vrt1.dataset.GetGeoTransform())
        self.assertEqual(vrt2.dataset.GetGCPProjection(), vrt1.dataset.GetGCPProjection())
        self.assertIn('filename', list(vrt2.dataset.GetMetadata().keys()))

    def test_copy_vrt_with_band(self):
        array = gdal.Open(self.test_file_gcps).ReadAsArray()[1, 10:, :]
        vrt1 = VRT.from_array(array)
        vrt2 = vrt1.copy()

        self.assertEqual(vrt2.dataset.RasterCount, 1)

    def test_copy_vrt_pixel_func(self):
        vrt1 = VRT()
        vrt1_xml = '''
        <VRTDataset rasterXSize="200" rasterYSize="200">
            <VRTRasterBand dataType="Byte" band="1">
                <ComplexSource>
                    <SourceFilename relativeToVRT="0">%s</SourceFilename>
                    <SourceBand>1</SourceBand>
                    <SourceProperties RasterXSize="200" RasterYSize="200" DataType="Byte" BlockXSize="200" BlockYSize="13" />
                    <SrcRect xOff="0" yOff="0" xSize="200" ySize="200" />
                    <DstRect xOff="0" yOff="0" xSize="200" ySize="200" />
                </ComplexSource>
            </VRTRasterBand>
            <VRTRasterBand dataType="Float32" band="2" subClass="VRTDerivedRasterBand">
                <ComplexSource>
                    <SourceFilename relativeToVRT="0">%s</SourceFilename>
                    <SourceBand>1</SourceBand>
                    <SourceProperties RasterXSize="200" RasterYSize="200" DataType="Byte" BlockXSize="128" BlockYSize="128" />
                    <SrcRect xOff="0" yOff="0" xSize="200" ySize="200" />
                    <DstRect xOff="0" yOff="0" xSize="200" ySize="200" />
                </ComplexSource>
                <PixelFunctionType>sqrt</PixelFunctionType>
            </VRTRasterBand>
        </VRTDataset>
       ''' % (self.test_file_gcps, vrt1.filename)
        vrt1.write_xml(vrt1_xml)
        vrt2 = vrt1.copy()

        self.assertFalse(os.path.basename(vrt1.filename) in vrt2.xml)

    def test_copy_geolocation(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt1 = VRT.from_lonlat(lon, lat)

        vrt2 = vrt1.copy()
        vrt1 = None
        self.assertTrue(vrt2.geolocation.x_vrt is not None)
        self.assertTrue(vrt2.geolocation.y_vrt is not None)

    def test_export(self):
        array = gdal.Open(self.test_file_gcps).ReadAsArray()[1, 10:, :]
        vrt = VRT.from_array(array)
        vrt.export(self.tmp_filename)
        self.assertTrue(self.tmp_filename)
        tree = ET.parse(self.tmp_filename)
        root = tree.getroot()

        self.assertEqual(root.tag, 'VRTDataset')
        self.assertIn('rasterXSize', list(root.keys()))
        self.assertIn('rasterYSize', list(root.keys()))

        self.assertEqual([e.tag for e in root], ['Metadata', 'VRTRasterBand'])

    def test_create_band(self):
        array = gdal.Open(self.test_file_gcps).ReadAsArray()[1, 10:, :]
        vrt1 = VRT.from_array(array)
        vrt2 = VRT(x_size=array.shape[1], y_size=array.shape[0])
        self.assertEqual(vrt2.dataset.RasterCount, 0)
        vrt2.create_band({'SourceFilename': vrt1.filename})
        self.assertEqual(vrt2.dataset.RasterCount, 1)

    def test_make_source_bands_xml(self):
        array = gdal.Open(self.test_file_gcps).ReadAsArray()[1, 10:, :]
        vrt1 = VRT.from_array(array)
        src1 = {'SourceFilename': vrt1.filename}
        src2 = VRT._make_source_bands_xml(src1)
        self.assertIn('XML', src2)
        self.assertEqual(src2['SourceFilename'], vrt1.filename)
        self.assertEqual(src2['SourceBand'], 1)
        self.assertEqual(src2['LUT'], '')
        self.assertEqual(src2['NODATA'], '')
        self.assertEqual(src2['SourceType'], 'ComplexSource')
        self.assertEqual(src2['ScaleRatio'], 1.0)
        self.assertEqual(src2['ScaleOffset'], 0.0)
        self.assertEqual(src2['DataType'], 1)
        self.assertEqual(src2['xSize'], 200)
        self.assertEqual(src2['ySize'], 190)

        with self.assertRaises(KeyError):
            src2 = VRT._make_source_bands_xml({})

    def test_set_add_band_options(self):
        # case 1
        srcs = [{'SourceFilename': 'filename', 'SourceBand': 1}]
        dst = []
        options = VRT._set_add_band_options(srcs, dst)
        self.assertEqual(options, [])
        # case 2
        srcs = [{'SourceFilename': 'filename',
                 'SourceBand': 0,
                 'ImageOffset': 0,
                 'PixelOffset': 0,
                 'LineOffset': 0,
                 'ByteOrder': 'i'}]
        options = VRT._set_add_band_options(srcs, dst)
        self.assertIn('subclass=VRTRawRasterBand', options)
        self.assertIn('SourceFilename=filename', options)
        self.assertIn('ImageOffset=0', options)
        self.assertIn('PixelOffset=0', options)
        self.assertIn('LineOffset=0', options)
        self.assertIn('ByteOrder=i', options)

    def test_remove_geotransform(self):
        ds = gdal.Open('NETCDF:"%s":UMass_AES' % self.test_file_arctic)
        vrt = VRT.copy_dataset(ds)
        self.assertTrue('<GeoTransform>' in vrt.xml)
        vrt._remove_geotransform()
        self.assertFalse('<GeoTransform>' in vrt.xml)

    def test_set_geotransform_for_resize(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt = VRT.from_lonlat(lon, lat)
        vrt._set_geotransform_for_resize()

        self.assertEqual(vrt.dataset.GetMetadata(str('GEOLOCATION')), {})
        self.assertEqual(vrt.dataset.GetGCPs(), ())
        self.assertEqual(vrt.dataset.GetGeoTransform(), (0.0, 1.0, 0.0, 30, 0.0, -1.0))

    def test_set_gcps_geolocation_geotransform_with_geolocation(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt = VRT.from_lonlat(lon, lat)
        vrt.create_band({str('SourceFilename'): vrt.geolocation.x_vrt.filename})
        vrt._set_gcps_geolocation_geotransform()
        self.assertFalse('<GeoTransform>' in vrt.xml)
        self.assertEqual(vrt.dataset.GetGCPs(), ())

    def test_set_gcps_geolocation_geotransform_with_gcps(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt = VRT.from_lonlat(lon, lat)
        vrt.create_band({'SourceFilename': vrt.geolocation.x_vrt.filename})
        vrt._remove_geolocation()
        vrt._set_gcps_geolocation_geotransform()
        self.assertFalse('<GeoTransform>' in vrt.xml)
        self.assertIsInstance(vrt.dataset.GetGCPs(), (list, tuple))
        self.assertTrue(len(vrt.dataset.GetGCPs()) > 0)
        self.assertEqual(vrt.dataset.GetMetadata(str('GEOLOCATION')), {})

    def test_set_gcps_geolocation_geotransform_with_geotransform(self):
        ds = gdal.Open('NETCDF:"%s":UMass_AES' % self.test_file_arctic)
        vrt = VRT.copy_dataset(ds)
        vrt._set_gcps_geolocation_geotransform()
        self.assertEqual(vrt.dataset.GetGeoTransform(),
                         (-1000000.0, 25000.0, 0.0, 5000000.0, 0.0, -25000.0))
        self.assertEqual(vrt.dataset.GetMetadata(str('GEOLOCATION')), {})
        self.assertEqual(vrt.dataset.GetGCPs(), ())

    def test_update_warped_vrt_xml(self):
        dataset = gdal.Open('NETCDF:"%s":UMass_AES' % self.test_file_arctic)
        warped_dataset = gdal.AutoCreateWarpedVRT(dataset, None, str(self.nsr_wkt), 0)
        warped_vrt = VRT.copy_dataset(warped_dataset)
        x_size = 100
        y_size = 200
        geo_transform = (0.0, 1.0, 0.0, 200.0, 0.0, -1.0)
        block_size = 64
        working_data_type = 'Float32'
        warped_vrt._update_warped_vrt_xml(x_size, y_size, geo_transform, block_size,
                                          working_data_type)

        self.assertEqual(warped_vrt.dataset.RasterXSize, x_size)
        self.assertEqual(warped_vrt.dataset.RasterYSize, y_size)
        self.assertEqual(warped_vrt.dataset.GetGeoTransform(), geo_transform)
        self.assertEqual(warped_vrt.dataset.GetRasterBand(1).GetBlockSize(),
                         [block_size, block_size])
        self.assertIn('<WorkingDataType>Float32</WorkingDataType>', warped_vrt.xml)

    def test_set_fake_gcps_empty(self):
        ds = gdal.Open('NETCDF:"%s":UMass_AES' % self.test_file_arctic)
        vrt = VRT.copy_dataset(ds)

        dst_wkt = vrt._set_fake_gcps(self.nsr_wkt, [], 1)
        self.assertEqual(dst_wkt, self.nsr_wkt)
        self.assertEqual(len(vrt.dataset.GetGCPs()), 0)

    def test_set_fake_gcps(self):
        ds = gdal.Open('NETCDF:"%s":UMass_AES' % self.test_file_arctic)
        gcps = gdal.Open(self.test_file_gcps).GetGCPs()
        vrt = VRT.copy_dataset(ds)

        dst_wkt = vrt._set_fake_gcps(self.nsr_wkt, gcps, 1)
        self.assertEqual(dst_wkt, None)
        self.assertEqual(len(vrt.dataset.GetGCPs()), len(gcps))
        self.assertEqual([gcp.GCPPixel for gcp in gcps],
                         [gcp.GCPX for gcp in vrt.dataset.GetGCPs()])
        self.assertEqual([gcp.GCPLine for gcp in gcps],
                         [gcp.GCPY for gcp in vrt.dataset.GetGCPs()])

    def test_get_dst_band_data_type(self):
        self.assertEqual(VRT._get_dst_band_data_type([], {'dataType': 'Float32'}), 'Float32')
        self.assertEqual(VRT._get_dst_band_data_type([1, 2, 3], {}), gdal.GDT_Float32)
        self.assertEqual(VRT._get_dst_band_data_type([{'ScaleRatio': 2}], {}), gdal.GDT_Float32)
        self.assertEqual(VRT._get_dst_band_data_type([{'LUT': [1, 2, 3]}], {}), gdal.GDT_Float32)
        self.assertEqual(VRT._get_dst_band_data_type([{}], {}), gdal.GDT_Float32)
        self.assertEqual(VRT._get_dst_band_data_type([{'DataType': 'Float32'}], {}), 'Float32')

    def test_create_band_name_no_wkv(self):
        self.mock_pti['get_wkv_variable'].side_effect = IndexError
        vrt = VRT()
        self.assertEqual(vrt._create_band_name({'name': 'name1'}), ('name1', {}))

    def test_create_band_name_wkv(self):
        short_name='sigma0'
        wkv = dict(short_name=short_name)
        self.mock_pti['get_wkv_variable'].return_value=wkv
        vrt = VRT()
        self.assertEqual(vrt._create_band_name({'wkv': short_name}), (short_name, wkv))
        self.assertEqual(vrt._create_band_name({'wkv': short_name, 'suffix': 'HH'}),
                         (short_name + '_HH', wkv))

    def test_create_band_name_existing_name(self):
        self.mock_pti['get_wkv_variable'].side_effect = IndexError
        vrt = VRT.from_array(np.zeros((10,10)))
        vrt.dataset.GetRasterBand(1).SetMetadata({'name':'band1'})
        self.assertEqual(vrt._create_band_name({'name': 'band1'}), ('band1_0000', {}))

    def test_create_band_name_wkv_and_name(self):
        name = 'some_name'
        wkv = dict(short_name='sigma0')
        self.mock_pti['get_wkv_variable'].return_value=wkv
        vrt = VRT()
        self.assertEqual(vrt._create_band_name({'wkv': 'sigma0', 'name': name}), (name, wkv))

    def test_leave_few_bands(self):
        ds = gdal.Open(self.test_file_gcps)
        vrt = VRT.copy_dataset(ds)
        vrt.leave_few_bands([1, 'L_469'])
        self.assertEqual(vrt.dataset.RasterCount,2)
        self.assertEqual(vrt.dataset.GetRasterBand(1).GetMetadataItem(str('name')), 'L_645')
        self.assertEqual(vrt.dataset.GetRasterBand(2).GetMetadataItem(str('name')), 'L_469')

    def test_find_complex_band(self):
        a = np.random.randn(100,100)
        vrt1 = VRT.from_array(a)
        vrt2 = VRT.from_array(a.astype(np.complex64))

        vrt3 = VRT.from_gdal_dataset(vrt1.dataset)
        vrt3.create_bands([{'src': {'SourceFilename': vrt1.filename}},
                           {'src': {'SourceFilename': vrt2.filename}}])

        self.assertEqual(vrt1._find_complex_band(), None)
        self.assertEqual(vrt2._find_complex_band(), 1)
        self.assertEqual(vrt3._find_complex_band(), 2)

    def test_split_complex_bands(self):
        a = np.random.randn(100,100)
        vrt1 = VRT.from_array(a.astype(np.complex64))
        vrt2 = VRT.from_array(a)
        vrt3 = VRT.from_array(a.astype(np.complex64))

        vrt4 = VRT.from_gdal_dataset(vrt1.dataset)
        vrt4.create_bands([{'src': {'SourceFilename': vrt1.filename}, 'dst': {'name': 'vrt1'}},
                           {'src': {'SourceFilename': vrt2.filename}, 'dst': {'name': 'vrt2'}},
                           {'src': {'SourceFilename': vrt3.filename}, 'dst': {'name': 'vrt3'}}])

        vrt4.split_complex_bands()

        self.assertEqual(vrt4.dataset.RasterCount,5)
        self.assertEqual(vrt4.dataset.GetRasterBand(1).GetMetadataItem(str('name')), 'vrt2')
        self.assertEqual(vrt4.dataset.GetRasterBand(2).GetMetadataItem(str('name')), 'vrt1_real')
        self.assertEqual(vrt4.dataset.GetRasterBand(3).GetMetadataItem(str('name')), 'vrt1_imag')
        self.assertEqual(vrt4.dataset.GetRasterBand(4).GetMetadataItem(str('name')), 'vrt3_real')
        self.assertEqual(vrt4.dataset.GetRasterBand(5).GetMetadataItem(str('name')), 'vrt3_imag')

    def test_create_geolocation_bands(self):
        lon, lat = np.meshgrid(np.linspace(0,5,10), np.linspace(10,20,30))
        vrt = VRT.from_lonlat(lon, lat)
        vrt.create_geolocation_bands()

        self.assertEqual(vrt.dataset.RasterCount, 2)
        self.assertEqual(vrt.dataset.GetRasterBand(1).GetMetadataItem(str('name')), 'longitude')
        self.assertEqual(vrt.dataset.GetRasterBand(2).GetMetadataItem(str('name')), 'latitude')
        self.assertTrue(np.allclose(vrt.dataset.GetRasterBand(1).ReadAsArray(), lon))
        self.assertTrue(np.allclose(vrt.dataset.GetRasterBand(2).ReadAsArray(), lat))

    def test_fix_band_metadata(self):
        ds = gdal.Open(self.test_file_gcps)
        vrt = VRT.copy_dataset(ds)
        self.assertIn('standard_name', vrt.dataset.GetRasterBand(1).GetMetadata())
        self.assertIn('time', vrt.dataset.GetRasterBand(1).GetMetadata())
        vrt.fix_band_metadata(['standard_name', 'time'])
        self.assertNotIn('standard_name', vrt.dataset.GetRasterBand(1).GetMetadata())
        self.assertNotIn('time', vrt.dataset.GetRasterBand(1).GetMetadata())

    def test_fix_global_metadata(self):
        ds = gdal.Open(self.test_file_gcps)
        vrt = VRT.copy_dataset(ds)
        vrt.dataset.SetMetadataItem(str('test'), str('"test"'))
        vrt.fix_global_metadata(['AREA_OR_POINT'])
        self.assertNotIn('AREA_OR_POINT', vrt.dataset.GetMetadata())
        self.assertEqual('&quot;test&quot;', vrt.dataset.GetMetadataItem(str('test')))

    def test_hardcopy_bands(self):
        ds = gdal.Open(self.test_file_gcps)
        vrt = VRT.copy_dataset(ds)
        vrt.hardcopy_bands()

        self.assertTrue(np.allclose(vrt.dataset.ReadAsArray(), ds.ReadAsArray()))
        band_nodes = Node.create(str(vrt.xml)).nodeList('VRTRasterBand')
        self.assertEqual(band_nodes[0].node('SourceFilename').value, vrt.band_vrts[1].filename)
        self.assertEqual(band_nodes[1].node('SourceFilename').value, vrt.band_vrts[2].filename)
        self.assertEqual(band_nodes[2].node('SourceFilename').value, vrt.band_vrts[3].filename)

    @patch.multiple(VRT, dataset=DEFAULT, __init__=Mock(return_value=None))
    def test_get_projection_dataset(self, dataset):
        proj = 'SOME_PROJECTION'
        dataset.GetProjection.return_value = proj
        dataset.GetGCPProjection.return_value = ''
        dataset.GetMetadata.return_value = {}

        vrt = VRT()
        proj_src = vrt.get_projection()
        self.assertEqual(proj_src, (proj, 'dataset'))

    @patch.multiple(VRT, dataset=DEFAULT, __init__=Mock(return_value=None))
    def test_get_projection_gcps(self, dataset):
        proj = 'SOME_PROJECTION'
        dataset.GetProjection.return_value = ''
        dataset.GetGCPProjection.return_value = proj
        dataset.GetMetadata.return_value = {}

        vrt = VRT()
        proj_src = vrt.get_projection()
        self.assertEqual(proj_src, (proj, 'gcps'))

    @patch.multiple(VRT, dataset=DEFAULT, __init__=Mock(return_value=None))
    def test_get_projection_geolocation(self, dataset):
        proj = 'SOME_PROJECTION'
        dataset.GetProjection.return_value = ''
        dataset.GetGCPProjection.return_value = ''
        dataset.GetMetadata.return_value = {'SRS': proj}

        vrt = VRT()
        proj_src = vrt.get_projection()
        self.assertEqual(proj_src, (proj, 'geolocation'))

    @patch.multiple(VRT, dataset=DEFAULT, __init__=Mock(return_value=None))
    def test_get_projection_raises_NansatProjectionError(self, dataset):
        dataset.GetProjection.return_value = ''
        dataset.GetGCPProjection.return_value = ''
        dataset.GetMetadata.return_value = {}

        vrt = VRT()
        with self.assertRaises(NansatProjectionError):
            proj = vrt.get_projection()

    def test_repr(self):
        # we mock the entire class [MagicMock(VRT)] and set instance attributes/methods
        mock_vrt1 = MagicMock(VRT, filename='aaa', vrt=None, __repr__=VRT.__repr__)
        mock_vrt2 = MagicMock(VRT, filename='bbb', vrt=mock_vrt1, __repr__=VRT.__repr__)

        # str(mock_vrt1) will call mock_vrt1.__repr__ and, hence, VRT.__repr__ which we test
        self.assertEqual(str(mock_vrt1), 'aaa')
        self.assertEqual(str(mock_vrt2), 'bbb=>aaa')

    @patch.multiple(VRT, create_band=DEFAULT, __init__=Mock(return_value=None))
    def test_add_swath_mask_band(self, create_band):
        vrt = VRT()
        vrt.filename = '/temp/filename.vrt'
        vrt._add_swath_mask_band()
        src = [{'SourceFilename': '/temp/filename.vrt',
                'SourceBand':  1,
                'DataType': 1}]
        dst ={'dataType': 1,
                'wkv': 'swath_binary_mask',
                'PixelFunctionType': 'OnesPixelFunc'}
        create_band.assert_called_once_with(src=src, dst=dst)

    def test_remove_strings_in_metadata_keys(self):
        gdal_metadata = {'aaa': 'bbb', 'NC_GLOBAL#ccc': 'ddd', 'NANSAT_eee': 'fff'}
        rm_strings = ['NC_GLOBAL#', 'NANSAT_']
        new_metadata = VRT._remove_strings_in_metadata_keys(gdal_metadata, rm_strings)
        self.assertEqual(new_metadata, {'aaa': 'bbb', 'ccc': 'ddd', 'eee': 'fff'})

    def test_super_vrt_of_geolocation_bands(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt1 = VRT.from_lonlat(lon, lat)
        vrt1.create_geolocation_bands()
        vrt2 = vrt1.get_super_vrt()
        self.assertTrue(hasattr(vrt2.vrt, 'vrt'))

    def test_get_shifted_vrt(self):
        deg = -10
        vrt1 = VRT.from_array(np.zeros((180,360)))
        vrt1.dataset.SetProjection(NSR().wkt)
        vrt2 = vrt1.get_shifted_vrt(deg)
        self.assertEqual(vrt1.dataset.GetGeoTransform()[0]+deg, vrt2.dataset.GetGeoTransform()[0])

    def test_get_super_vrt(self):
        ds = gdal.Open(self.test_file_gcps)
        vrt1 = VRT.from_gdal_dataset(ds, metadata=ds.GetMetadata())
        vrt2 = vrt1.get_super_vrt()
        self.assertIsInstance(vrt2.vrt, VRT)
        self.assertEqual(vrt2.dataset.GetMetadataItem(str('AREA_OR_POINT')), 'Area')

    def test_get_super_vrt_geolocation(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt1 = VRT.from_lonlat(lon, lat)
        vrt2 = vrt1.get_super_vrt()
        vrt1 = None
        self.assertTrue(vrt2.geolocation.x_vrt is not None)
        self.assertTrue(vrt2.geolocation.y_vrt is not None)

    def test_get_super_vrt_and_copy(self):
        array = np.zeros((10,10))
        vrt = VRT.from_array(array)
        vrt = vrt.get_super_vrt()
        vrt = vrt.copy()
        data = vrt.dataset.ReadAsArray()

        self.assertFalse(data is None)
        self.assertTrue(np.all(data == array))

    def test_get_sub_vrt0(self):
        vrt1 = VRT()
        vrt2 = vrt1.get_sub_vrt()
        self.assertEqual(vrt1, vrt2)

    def test_get_sub_vrt3(self):
        vrt1 = VRT().get_super_vrt().get_super_vrt().get_super_vrt()
        vrt2 = vrt1.get_sub_vrt(3)
        self.assertEqual(vrt2.vrt, None)

    def test_get_sub_vrt_steps_0(self):
        vrt1 = VRT().get_super_vrt()
        vrt2 = vrt1.get_sub_vrt(steps=0)
        self.assertEqual(vrt1, vrt2)

    def test_transform_points(self):
        ds = gdal.Open(self.test_file_gcps)
        vrt1 = VRT.from_gdal_dataset(ds, metadata=ds.GetMetadata())
        vrt1.tps = True
        lon, lat = vrt1.transform_points([1, 2, 3], [4, 5, 6])
        self.assertTrue(np.allclose(lon, np.array([28.23549571, 28.24337106, 28.25126129])))
        self.assertTrue(np.allclose(lat, np.array([71.52509848, 71.51913744, 71.51317568])))
        lon, lat = vrt1.transform_points([], [])
        self.assertTrue(np.allclose(lon, np.array([])))
        self.assertTrue(np.allclose(lat, np.array([])))

    def test_make_filename(self):
        filename1 = VRT._make_filename()
        filename2 = VRT._make_filename(extention='smth')
        filename3 = VRT._make_filename(nomem=True)
        self.assertTrue(filename1.startswith('/vsimem/'))
        self.assertTrue(filename2.startswith('/vsimem/'))
        self.assertTrue(filename2.endswith('.smth'))
        self.assertTrue(os.path.exists(filename3))

    def test_transform_coordinates_list(self):
        src_srs = NSR()
        dst_srs = NSR(str('+proj=stere'))
        src_points = ([1,2,3,4],[5,6,7,8])
        dst_x, dst_y, dst_z = VRT.transform_coordinates(src_srs, src_points, dst_srs)
        # check if shape of the result matches the expected shape (list with four points)
        self.assertEqual(dst_x.shape, (4,))
        self.assertEqual(dst_y.shape, (4,))
        self.assertEqual(dst_z.shape, (4,))

    def test_transform_coordinates_1d_array(self):
        src_srs = NSR()
        dst_srs = NSR(str('+proj=stere'))
        src_points = (np.array([1,2,3,4]), np.array([5,6,7,8]), np.array([5,6,7,8]))
        dst_x, dst_y, dst_z = VRT.transform_coordinates(src_srs, src_points, dst_srs)
        # check if shape of the result matches the expected shape (list with four points)
        self.assertEqual(dst_x.shape, (4,))
        self.assertEqual(dst_y.shape, (4,))
        self.assertEqual(dst_z.shape, (4,))

    def test_transform_coordinates_2d_array(self):
        src_srs = NSR()
        dst_srs = NSR(str('+proj=stere'))
        src_points = (np.array([[1,2,3,4],[1,2,3,4]]),
                      np.array([[5,6,7,8],[5,6,7,8]]),
                      np.array([[5,6,7,8],[5,6,7,8]]),)
        dst_x, dst_y, dst_z = VRT.transform_coordinates(src_srs, src_points, dst_srs)
        # check if shape of the result matches the expected shape (2x4 array)
        self.assertEqual(dst_x.shape, (2,4))
        self.assertEqual(dst_y.shape, (2,4))
        self.assertEqual(dst_z.shape, (2,4))

    def test_reproject_gcps(self):
        lon, lat = np.meshgrid(np.linspace(0, 5, 10), np.linspace(10, 20, 30))
        vrt1 = VRT.from_lonlat(lon, lat)
        vrt1.reproject_gcps(str('+proj=cea'))
        self.assertIn('Cylindrical_Equal_Area', vrt1.dataset.GetGCPProjection())
        self.assertEqual(round(vrt1.dataset.GetGCPs()[0].GCPX), 0)
        self.assertEqual(round(vrt1.dataset.GetGCPs()[0].GCPY), 1100286)
        self.assertEqual(round(vrt1.dataset.GetGCPs()[-1].GCPX), 556597)
        self.assertEqual(round(vrt1.dataset.GetGCPs()[-1].GCPY), 2096057)

if __name__ == "__main__":
    unittest.main()



# Name:        mapper_aster_l1a
# Purpose:     Mapping for L2 data from the OBPG web-site
# Authors:      Anton Korosov
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html
from datetime import datetime, timedelta
from math import ceil
from dateutil.parser import parse

from nansat.utils import gdal, ogr
from nansat.exceptions import WrongMapperError
from nansat.vrt import VRT
from nansat.nsr import NSR


class Mapper(VRT):
    ''' Mapper for ASTER L1A VNIR data'''

    def __init__(self, filename, gdalDataset, gdalMetadata,
                 GCP_COUNT=10,
                 bandNames=['VNIR_Band1', 'VNIR_Band2', 'VNIR_Band3N'],
                 bandWaves=[560, 660, 820], **kwargs):
        ''' Create VRT

        Parameters
        -----------
        GCP_COUNT : int
            number of GCPs along each dimention
        bandNames : list of string (band name)
        bandWaves : list of integer (waves corresponding to band name)

        Band name and waves
        --------------------
        'VNIR_Band3B' : 820, 'SWIR_Band4' : 1650, 'SWIR_Band5' : 2165,
        'SWIR_Band6' : 2205, 'SWIR_Band7' : 2260, 'SWIR_Band8' : 2330,
        'SWIR_Band9' : 2395, 'TIR_Band10' : 8300, 'TIR_Band11' : 8650,
        'TIR_Band12' : 9100, 'TIR_Band13' : 10600, 'TIR_Band14' : 11300

        '''
        # check if it is ASTER L1A
        try:
            assert 'AST_L1A_' in filename
            shortName = gdalMetadata['INSTRUMENTSHORTNAME']
            assert shortName == 'ASTER'
        except:
            raise WrongMapperError

        subDatasets = gdalDataset.GetSubDatasets()

        # find datasets for each band and generate metaDict
        metaDict = []
        bandDatasetMask = 'HDF4_EOS:EOS_SWATH:"%s":%s:ImageData'
        for bandName, bandWave in zip(bandNames, bandWaves):
            metaEntry = {'src': {'SourceFilename': (bandDatasetMask
                                                    % (filename, bandName)),
                                 'SourceBand': 1,
                                 'DataType': 6,
                                 },
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': str(bandWave),
                                 'suffix': str(bandWave),
                                 }}
            metaDict.append(metaEntry)

        # create empty VRT dataset with geolocation only
        gdalSubDataset = gdal.Open(metaDict[0]['src']['SourceFilename'])
        self._init_from_gdal_dataset(gdalSubDataset, metadata=gdalSubDataset.GetMetadata())

        # add bands with metadata and corresponding values to the empty VRT
        self.create_bands(metaDict)

        # find largest lon/lat subdatasets
        latShape0 = 0
        for subDataset in subDatasets:
            if 'Latitude' in subDataset[1]:
                ls = int(subDataset[1].strip().split('[')[1].split('x')[0])
                if ls >= latShape0:
                    latShape0 = ls
                    latSubDS = subDataset[0]
            if 'Longitude' in subDataset[1]:
                ls = int(subDataset[1].strip().split('[')[1].split('x')[0])
                if ls >= latShape0:
                    latShape0 = ls
                    lonSubDS = subDataset[0]
        self.logger.debug(latSubDS)
        self.logger.debug(lonSubDS)

        # get lat/lon matrices
        xDataset = gdal.Open(lonSubDS)
        yDataset = gdal.Open(latSubDS)

        longitude = xDataset.ReadAsArray()
        latitude = yDataset.ReadAsArray()

        step0 = longitude.shape[0] / GCP_COUNT
        step1 = longitude.shape[1] / GCP_COUNT

        # estimate pixel/line step
        pixelStep = int(ceil(float(gdalSubDataset.RasterXSize) /
                             float(xDataset.RasterXSize)))
        lineStep = int(ceil(float(gdalSubDataset.RasterYSize) /
                            float(xDataset.RasterYSize)))
        self.logger.debug('steps: %d %d %d %d' % (step0, step1,
                                                  pixelStep, lineStep))

        # generate list of GCPs
        gcps = []
        k = 0
        for i0 in range(0, latitude.shape[0], step0):
            for i1 in range(0, latitude.shape[1], step1):
                # create GCP with X,Y,pixel,line from lat/lon matrices
                lon = float(longitude[i0, i1])
                lat = float(latitude[i0, i1])
                if (lon >= -180 and lon <= 180 and lat >= -90 and lat <= 90):
                    gcp = gdal.GCP(lon, lat, 0, i1 * pixelStep, i0 * lineStep)
                    self.logger.debug('%d %d %d %f %f' % (k, gcp.GCPPixel,
                                                          gcp.GCPLine,
                                                          gcp.GCPX, gcp.GCPY))
                    gcps.append(gcp)
                    k += 1
        # append GCPs and lat/lon projection to the vsiDataset
        self.dataset.SetGCPs(gcps, NSR().wkt)

        # Adding valid time to dataset
        self.dataset.SetMetadataItem('time_coverage_start',
                                     parse(gdalMetadata['FIRSTPACKETTIME']).isoformat())
        self.dataset.SetMetadataItem('time_coverage_end',
                                     parse(gdalMetadata['LASTPACKETTIME']).isoformat())

        mm = pti.get_gcmd_instrument('ASTER')
        ee = pti.get_gcmd_platform('TERRA')
        self.dataset.SetMetadataItem('instrument', json.dumps(mm))
        self.dataset.SetMetadataItem('platform', json.dumps(ee))



#------------------------------------------------------------------------------
# Name:     mapper_sentinel1_l1.py
# Purpose:
#
# Author:       Morten Wergeland Hansen, Anton Korosov
#
# Created:  12.09.2014
# Copyright:    (c) NERSC
# License: GPL V3
#------------------------------------------------------------------------------
import warnings

import os
import glob
import zipfile
import numpy as np
from dateutil.parser import parse
import xml.etree.ElementTree as ET

try:
    import scipy
except:
    IMPORT_SCIPY = False
else:
    IMPORT_SCIPY = True

import json
import pythesint as pti

from nansat.vrt import VRT
from nansat.utils import gdal, initial_bearing
from nansat.exceptions import WrongMapperError, NansatReadError
from nansat.nsr import NSR
from nansat.node import Node


class Mapper(VRT):
    """ Create VRT with mapping of Sentinel-1 (A and B) stripmap mode (S1A_SM)

    Parameters
    ----------
    filename : str
        name of input Sentinel-1 L1 file
    gdalDataset : None
    gdalMetadata : None
    fast : bool
        Flag that triggers faster reading of metadata from Sentinel-1 file.
        If True, no bands are added to the dataset and georeference is not corrected.
        If False, all bands are added and GCPs are corrected if necessary
        (see Mapper.correct_geolocation_data for details).

    Note
    ----
    Creates self.dataset and populates it with S1 bands (when fast=False).
    """
    def __init__(self, filename, gdalDataset, gdalMetadata, fast=False, fixgcp=True, **kwargs):
        if not os.path.split(filename.rstrip('/'))[1][:3] in ['S1A', 'S1B']:
            raise WrongMapperError('%s: Not Sentinel 1A or 1B' %filename)

        if not IMPORT_SCIPY:
            raise NansatReadError('Sentinel-1 data cannot be read because scipy is not installed')

        if zipfile.is_zipfile(filename):
            zz = zipfile.PyZipFile(filename)
            # Assuming the file names are consistent, the polarization
            # dependent data should be sorted equally such that we can use the
            # same indices consistently for all the following lists
            # THIS IS NOT THE CASE...
            mds_files = ['/vsizip/%s/%s' % (filename, fn)
                        for fn in zz.namelist() if 'measurement/s1' in fn]
            calibration_files = ['/vsizip/%s/%s' % (filename, fn)
                        for fn in zz.namelist()
                        if 'annotation/calibration/calibration-s1' in fn]
            noise_files = ['/vsizip/%s/%s' % (filename, fn)
                          for fn in zz.namelist()
                          if 'annotation/calibration/noise-s1' in fn]
            annotation_files = ['/vsizip/%s/%s' % (filename, fn)
                               for fn in zz.namelist()
                               if 'annotation/s1' in fn]
            manifest_files = ['/vsizip/%s/%s' % (filename, fn)
                            for fn in zz.namelist()
                            if 'manifest.safe' in fn]
            zz.close()
        else:
            mds_files = glob.glob('%s/measurement/s1*' % filename)
            calibration_files = glob.glob('%s/annotation/calibration/calibration-s1*'
                                 % filename)
            noise_files = glob.glob('%s/annotation/calibration/noise-s1*'
                                   % filename)
            annotation_files = glob.glob('%s/annotation/s1*'
                                        % filename)
            manifest_files = glob.glob('%s/manifest.safe' % filename)

        if (not mds_files or not calibration_files or not noise_files or
            not annotation_files or not manifest_files):
            raise WrongMapperError(filename)

        # convert list of MDS files into dictionary. Keys - polarizations in upper case.
        mds_files = {os.path.basename(ff).split('-')[3].upper():ff for ff in mds_files}
        polarizations = list(mds_files.keys())

        # read annotation files
        self.annotation_data = self.read_annotation(annotation_files)
        if not fast and fixgcp:
            self.correct_geolocation_data()

        # read manifest file
        manifest_data = self.read_manifest_data(manifest_files[0])

        # very fast constructor without any bands only with some metadata and geolocation
        self._init_empty(manifest_data, self.annotation_data)

        # skip adding bands in the fast mode and RETURN
        if fast:
            return

        # Open data files with GDAL
        gdalDatasets = {}
        for pol in polarizations:
            gdalDatasets[pol] = gdal.Open(mds_files[pol])

            if not gdalDatasets[pol]:
                raise WrongMapperError('%s: No Sentinel-1 datasets found' % mds_files[pol])

        # Check metadata to confirm it is Sentinel-1 L1
        metadata = gdalDatasets[polarizations[0]].GetMetadata()

        # create full size VRTs with incidenceAngle and elevationAngle
        annotation_vrts = self.vrts_from_arrays(self.annotation_data,
                                                ['incidenceAngle', 'elevationAngle'])
        self.band_vrts.update(annotation_vrts)

        # create full size VRTS with calibration LUT
        calibration_names = ['sigmaNought', 'betaNought']
        calibration_list_tag = 'calibrationVectorList'
        for calibration_file in calibration_files:
            pol = '_' + os.path.basename(calibration_file).split('-')[4].upper()
            xml = self.read_vsi(calibration_file)
            calibration_data = self.read_calibration(xml, calibration_list_tag, calibration_names, pol)
            calibration_vrts = self.vrts_from_arrays(calibration_data, calibration_names, pol, True, 1)
            self.band_vrts.update(calibration_vrts)

        # create full size VRTS with noise LUT
        for noise_file in noise_files:
            pol = '_' + os.path.basename(noise_file).split('-')[4].upper()
            xml = self.read_vsi(noise_file)
            if '<noiseVectorList' in xml:
                noise_list_tag = 'noiseVectorList'
                noise_name = 'noiseLut'
            elif '<noiseRangeVectorList' in xml:
                noise_list_tag = 'noiseRangeVectorList'
                noise_name = 'noiseRangeLut'
            noise_data = self.read_calibration(xml, noise_list_tag, [noise_name], pol)
            noise_vrts = self.vrts_from_arrays(noise_data, [noise_name], pol, True, 1)
            self.band_vrts.update(noise_vrts)

        #### Create metaDict: dict with metadata for all bands
        metaDict = []
        bandNumberDict = {}
        bnmax = 0
        for pol in polarizations:
            dsPath, dsName = os.path.split(mds_files[pol])
            name = 'DN_%s' % pol
            # A dictionary of band numbers is needed for the pixel function
            # bands further down. This is not the best solution. It would be
            # better to have a function in VRT that returns the number given a
            # band name. This function exists in Nansat but could perhaps be
            # moved to VRT? The existing nansat function could just call the
            # VRT one...
            bandNumberDict[name] = bnmax + 1
            bnmax = bandNumberDict[name]
            band = gdalDatasets[pol].GetRasterBand(1)
            dtype = band.DataType
            metaDict.append({
                'src': {
                    'SourceFilename': mds_files[pol],
                    'SourceBand': 1,
                    'DataType': dtype,
                },
                'dst': {
                    'name': name,
                },
            })
        # add bands with metadata and corresponding values to the empty VRT
        self.create_bands(metaDict)

        '''
        Calibration should be performed as

        s0 = DN^2/sigmaNought^2,

        where sigmaNought is from e.g.
        annotation/calibration/calibration-s1a-iw-grd-hh-20140811t151231-20140811t151301-001894-001cc7-001.xml,
        and DN is the Digital Numbers in the tiff files.

        Also the noise should be subtracted.

        See
        https://sentinel.esa.int/web/sentinel/sentinel-1-sar-wiki/-/wiki/Sentinel%20One/Application+of+Radiometric+Calibration+LUT

        The noise correction/subtraction is implemented in an independent package "sentinel1denoised"
        See
        https://github.com/nansencenter/sentinel1denoised
        '''

        # Get look direction
        longitude, latitude = self.transform_points(calibration_data['pixel'].flatten(),
                                                    calibration_data['line'].flatten())
        longitude.shape = calibration_data['pixel'].shape
        latitude.shape = calibration_data['pixel'].shape
        sat_heading = initial_bearing(longitude[:-1, :],
                                      latitude[:-1, :],
                                      longitude[1:, :],
                                      latitude[1:, :])
        look_direction = scipy.ndimage.interpolation.zoom(
            np.mod(sat_heading + 90, 360),
            (np.shape(longitude)[0] / (np.shape(longitude)[0]-1.), 1))

        # Decompose, to avoid interpolation errors around 0 <-> 360
        look_direction_u = np.sin(np.deg2rad(look_direction))
        look_direction_v = np.cos(np.deg2rad(look_direction))
        look_u_VRT = VRT.from_array(look_direction_u)
        look_v_VRT = VRT.from_array(look_direction_v)
        lookVRT = VRT.from_lonlat(longitude, latitude)
        lookVRT.create_band([{'SourceFilename': look_u_VRT.filename,
                               'SourceBand': 1},
                              {'SourceFilename': look_v_VRT.filename,
                               'SourceBand': 1}],
                             {'PixelFunctionType': 'UVToDirectionTo'}
                             )

        # Blow up to full size
        lookVRT = lookVRT.get_resized_vrt(self.dataset.RasterXSize, self.dataset.RasterYSize, 1)

        # Store VRTs so that they are accessible later
        self.band_vrts['look_u_VRT'] = look_u_VRT
        self.band_vrts['look_v_VRT'] = look_v_VRT
        self.band_vrts['lookVRT'] = lookVRT

        metaDict = []
        # Add bands to full size VRT
        for pol in polarizations:
            name = 'sigmaNought_%s' % pol
            bandNumberDict[name] = bnmax+1
            bnmax = bandNumberDict[name]
            metaDict.append(
                {'src': {'SourceFilename':
                         (self.band_vrts[name].filename),
                         'SourceBand': 1
                         },
                 'dst': {'name': name
                         }
                 })
            name = 'noise_%s' % pol
            bandNumberDict[name] = bnmax+1
            bnmax = bandNumberDict[name]
            metaDict.append({
                'src': {
                    'SourceFilename': self.band_vrts['%s_%s' % (noise_name, pol)].filename,
                    'SourceBand': 1
                },
                'dst': {
                    'name': name
                }
            })

        name = 'look_direction'
        bandNumberDict[name] = bnmax+1
        bnmax = bandNumberDict[name]
        metaDict.append({
            'src': {
                'SourceFilename': self.band_vrts['lookVRT'].filename,
                'SourceBand': 1
            },
            'dst': {
                'wkv': 'sensor_azimuth_angle',
                'name': name
            }
        })

        for pol in polarizations:
            dsPath, dsName = os.path.split(mds_files[pol])
            name = 'sigma0_%s' % pol
            bandNumberDict[name] = bnmax+1
            bnmax = bandNumberDict[name]
            metaDict.append(
                {'src': [{'SourceFilename': self.filename,
                          'SourceBand': bandNumberDict['DN_%s' % pol],
                          },
                         {'SourceFilename': self.band_vrts['sigmaNought_%s' % pol].filename,
                          'SourceBand': 1
                          }
                         ],
                 'dst': {'wkv': 'surface_backwards_scattering_coefficient_of_radar_wave',
                         'PixelFunctionType': 'Sentinel1Calibration',
                         'polarization': pol,
                         'suffix': pol,
                         },
                 })
            name = 'beta0_%s' % pol
            bandNumberDict[name] = bnmax+1
            bnmax = bandNumberDict[name]
            metaDict.append(
                {'src': [{'SourceFilename': self.filename,
                          'SourceBand': bandNumberDict['DN_%s' % pol]
                          },
                         {'SourceFilename': self.band_vrts['betaNought_%s' % pol].filename,
                          'SourceBand': 1
                          }
                         ],
                 'dst': {'wkv': 'surface_backwards_brightness_coefficient_of_radar_wave',
                         'PixelFunctionType': 'Sentinel1Calibration',
                         'polarization': pol,
                         'suffix': pol,
                         },
                 })

        self.create_bands(metaDict)

        # Add incidence angle as band
        name = 'incidence_angle'
        bandNumberDict[name] = bnmax+1
        bnmax = bandNumberDict[name]
        src = {'SourceFilename': self.band_vrts['incidenceAngle'].filename,
               'SourceBand': 1}
        dst = {'wkv': 'angle_of_incidence',
               'name': name}
        self.create_band(src, dst)
        self.dataset.FlushCache()

        # Add elevation angle as band
        name = 'elevation_angle'
        bandNumberDict[name] = bnmax+1
        bnmax = bandNumberDict[name]
        src = {'SourceFilename': self.band_vrts['elevationAngle'].filename,
               'SourceBand': 1}
        dst = {'wkv': 'angle_of_elevation',
               'name': name}
        self.create_band(src, dst)
        self.dataset.FlushCache()

        # Add sigma0_VV
        if 'VV' not in polarizations and 'HH' in polarizations:
            name = 'sigma0_VV'
            bandNumberDict[name] = bnmax+1
            bnmax = bandNumberDict[name]
            src = [{'SourceFilename': self.filename,
                    'SourceBand': bandNumberDict['DN_HH'],
                    },
                   {'SourceFilename': (self.band_vrts['sigmaNought_HH'].
                                       filename),
                    'SourceBand': 1,
                    },
                   {'SourceFilename': self.band_vrts['incidenceAngle'].filename,
                    'SourceBand': 1}
                   ]
            dst = {'wkv': 'surface_backwards_scattering_coefficient_of_radar_wave',
                   'PixelFunctionType': 'Sentinel1Sigma0HHToSigma0VV',
                   'polarization': 'VV',
                   'suffix': 'VV'}
            self.create_band(src, dst)
            self.dataset.FlushCache()


    def read_calibration(self, xml, vectorListName, variable_names, pol):
        """ Read calibration data from calibration or noise XML files
        Parameters
        ----------
        xml : str
            String with XML from calibration or noise files
        vectorListName : str
            tag of the element that contains lists with LUT values
        variable_names : list of str
            names of LUT variable to read
        pol : str
            HH, HV, etc

        Returns
        -------
        data : dict
            Calibration or noise data. Keys:
            The same as variable_names + 'pixel', 'line'
        """
        data = {}
        n = Node.create(xml)
        vecList = n.node(vectorListName)
        data['pixel'] = []
        data['line'] = []
        for var_name in variable_names:
            data[var_name+pol] = []
        xLengths = []
        for vec in vecList.children:
            xVec = list(map(int, vec['pixel'].split()))
            xLengths.append(len(xVec))
            data['pixel'].append(xVec)
            data['line'].append(int(vec['line']))
            for var_name in variable_names:
                data[var_name+pol].append(np.fromiter(vec[var_name].split(), float))

        # truncate data['pixel'] and var_name to minimum length for all rows
        minLength = np.min(xLengths)
        data['pixel'] = [x[:minLength] for x in data['pixel']]
        for var_name in variable_names:
            data[var_name+pol] = [d[:minLength] for d in data[var_name+pol]]

        data['pixel'] = np.array(data['pixel'])
        for var_name in variable_names:
            data[var_name+pol] = np.array(data[var_name+pol])
        data['line'] = np.array([data['line'], ]*np.shape(data['pixel'])[1]).transpose()

        return data

    def read_annotation(self, annotation_files):
        """ Read lon, lat, etc from annotation XML

        Parameters
        ----------
        annotation_files : list
            strings with names of annotation files

        Returns
        -------
        data : dict
            geolocation data from the XML as 2D np.arrays. Keys:
                pixel, line, longitude, latitude, height, incidenceAngle, elevationAngle: 2D arrays
                shape : tuple (shape of geolocation data arrays)
                x_size, y_size : int
                pol : list

        """
        variable_names = ['pixel', 'line', 'longitude', 'latitude', 'height', 'incidenceAngle', 'elevationAngle']
        data = {var_name:[] for var_name in variable_names}

        xml = self.read_vsi(annotation_files[0])
        xml = Node.create(xml)
        geolocation_points = xml.node('geolocationGrid').node('geolocationGridPointList').children
        # collect data from XML into dictionary with lists
        for point in geolocation_points:
            for var_name in variable_names:
                data[var_name].append(point[var_name])

        # convert lists to 1D arrays
        for var_name in variable_names:
            data[var_name] = np.fromiter(data[var_name], float)

        # get shape of geolocation matrix (number of occurence of minimal element)
        data['shape'] = (data['pixel']==0).sum(), (data['line']==0).sum()

        # convert 1D arrays to 2D
        for var_name in variable_names:
            data[var_name].shape = data['shape']

        # get raster dimentions
        image_info = xml.node('imageAnnotation').node('imageInformation')
        data['x_size'] = int(image_info.node('numberOfSamples').value)
        data['y_size'] = int(image_info.node('numberOfLines').value)

        # get list of polarizations
        data['pol'] = []
        for ff in annotation_files:
            p = os.path.basename(ff).split('-')[3]
            data['pol'].append(p.upper())

        return data

    def _init_empty(self, manifest_data, annotation_data):
        """ Fast initialization from minimum of information

        Parameters
        ----------
        manifest_data : dict
            data from the manifest file (time_coverage_start, etc)
        annotation_data : dict
            data from annotation file (longitude, latitude, x_size, etc)

        Note
        ----
            Calls VRT.__init__, Adds GCPs, metadata
        """
        # init empty dataset
        super(Mapper, self).__init__(annotation_data['x_size'], annotation_data['y_size'])
        # add GCPs from (corrected) geolocation data
        gcps = Mapper.create_gcps(annotation_data['longitude'],
                                  annotation_data['latitude'],
                                  annotation_data['height'],
                                  annotation_data['pixel'],
                                  annotation_data['line'])
        self.dataset.SetGCPs(gcps, NSR().wkt)
        # set metadata
        self.dataset.SetMetadataItem('time_coverage_start', manifest_data['time_coverage_start'])
        self.dataset.SetMetadataItem('time_coverage_end', manifest_data['time_coverage_end'])
        platform_name = manifest_data['platform_family_name'] + manifest_data['platform_number']
        self.dataset.SetMetadataItem('platform', json.dumps(pti.get_gcmd_platform(platform_name)))
        self.dataset.SetMetadataItem('instrument', json.dumps(pti.get_gcmd_instrument('SAR')))
        self.dataset.SetMetadataItem('entry_title', platform_name + ' SAR')
        self.dataset.SetMetadataItem('data_center', json.dumps(pti.get_gcmd_provider('ESA/EO')))
        self.dataset.SetMetadataItem('iso_topic_category', json.dumps(pti.get_iso19115_topic_category('Oceans')))
        self.dataset.SetMetadataItem('summary', platform_name + ' SAR data')
        self.dataset.FlushCache()

    def correct_geolocation_data(self):
        """ Correct lon/lat values in geolocation data for points high above ground (incorrect)

        Each GCP in Sentinel-1 L1 image (both in the GeoTIF files and Annotation LUT) have five
        coordinates: X, Y, Z (height), Pixel and Line. On some scenes that cover Greenland (and
        probably other lands) some GCPs have height above zero even over ocean. This is incorrect,
        because the radar signal comes actually from the surface and not from a point above the
        ground as stipulated in such GCPs. Correction of GCPs in this function is equivalelnt to
        reverse DEM correction of SAR data.

        Notes
        -------
        Updates 'pixel' and 'height' in self.annotation_data

        """
        PIXEL_SIZE = 40. # meters
        pix_corr = (self.annotation_data['height'] /
                    np.tan(np.radians(self.annotation_data['incidenceAngle'])) / PIXEL_SIZE)
        self.annotation_data['pixel'] += pix_corr
        self.annotation_data['height'][:] = 0

    @staticmethod
    def create_gcps(x, y, z, p, l):
        """ Create GCPs from geolocation data

        Parameters
        ----------
        x, y, z, p, l
        N-D arrays with value of X, Y, Z, Pixel and Line coordinates.
        X and Y are typically lon, lat, Z - height.

        Returns
        -------
        gcps : list with GDAL GCPs

        """
        gcps = []
        for xi, yi, zi, pi, li in zip(x.flat, y.flat, z.flat, p.flat, l.flat):
            gcps.append(gdal.GCP(xi, yi, zi, pi, li))
        return gcps

    def read_manifest_data(self, input_file):
        """ Read information (time_coverage_start, etc) manifest XML

        Parameters
        ----------
        input_file : str
            name of manifest file

        Returns
        -------
        data : dict
            manifest data. Keys:
                time_coverage_start
                time_coverage_end
                platform_familyName
                platform_number

        """

        data = {}
        xml = self.read_vsi(input_file)
        # set time as acquisition start time
        n = Node.create(xml)
        meta = n.node('metadataSection')
        for nn in meta.children:
            if str(nn.getAttribute('ID')) == 'acquisitionPeriod':
                # get valid time
                data['time_coverage_start'] = parse((nn.node('metadataWrap').
                                                     node('xmlData').
                                                     node('safe:acquisitionPeriod')['safe:startTime']
                                                     )).isoformat()
                data['time_coverage_end'] = parse((nn.node('metadataWrap').
                                                   node('xmlData').
                                                   node('safe:acquisitionPeriod')['safe:stopTime']
                                                   )).isoformat()
            if str(nn.getAttribute('ID')) == 'platform':
                data['platform_family_name'] = str(nn.node('metadataWrap').
                                                     node('xmlData').
                                                     node('safe:platform')['safe:familyName'])
                data['platform_number'] = str(nn.node('metadataWrap').
                                                     node('xmlData').
                                                     node('safe:platform')['safe:number'])
        return data

    def vrts_from_arrays(self, data, variable_names, pol='', resize=True, resample_alg=2):
        """ Convert input dict with arrays into dict with VRTs

        Parameters
        ----------
        data : dict
            2D arrays with data from LUT
        variable_names : list of str
            variable names that should be converted to VRTs
        pol : str
            HH, HV, etc
        resize : bool
            Shall VRT be zoomed to full size?
        resample_alg : int
            Index of resampling algorithm. See VRT.get_resized_vrt()

        Returns
        -------
        vrts : dict with (resized) VRTs

        """
        vrts = {}
        for var_name in variable_names:
            vrts[var_name+pol] = VRT.from_array(data[var_name+pol])
            if resize:
                vrts[var_name+pol] = vrts[var_name+pol].get_resized_vrt(self.dataset.RasterXSize,
                                                                        self.dataset.RasterYSize,
                                                                        resample_alg)
        return vrts



# ------------------------------------------------------------------------------
# Name:     test_radarsat2.py
# Purpose:      End to end testing of Nansat for Radarsat-2
#
# Author:   Morten Wergeland Hansen
# Modified:	Morten Wergeland Hansen, Aleksander Vines
#
# Created:  17.04.2015
# Last modified:23.12.2015 13:21
# Copyright:    (c) NERSC
# License:
# ------------------------------------------------------------------------------
import os
import sys
import numpy as np

from nansat.nansat import Nansat
from nansat_integration_tests.mapper_test_archive import DataForTestingMappers


class TestRadarsat(object):

    def test_all_rs2_files(self):
        sys.stderr.write('\ntest_all_rs2_files\n')
        testData = DataForTestingMappers()
        rs2Index = [i for i,
                    v in enumerate(testData.mapperData) if v['mapperName'] == 'radarsat2']
        for index in rs2Index:
            rsfile = testData.mapperData[index][0]
            # yield self.export2thredds, rsfile
            yield self.export, rsfile
            yield self.incidence_angle, rsfile
            yield self.export_band, rsfile
            yield self.resize, rsfile

    def export2thredds(self, rsfile):
        sys.stderr.write('\nexport2thredds:'+rsfile)
        ncfile = 'test.nc'
        orig = Nansat(rsfile)
        orig.export2thredds(ncfile, bands={'incidence_angle': {}})
        copy = Nansat(ncfile)
        inc0 = orig['incidence_angle']
        inc1 = copy['incidence_angle']
        np.testing.assert_allclose(inc0, inc1)
        os.unlink(ncfile)

    def export(self, rsfile):
        sys.stderr.write('\nexport:'+rsfile+'\n')
        ncfile = 'test.nc'
        orig = Nansat(rsfile)
        sys.stderr.write('\nExporting\n')
        orig.export(ncfile)
        sys.stderr.write('\nOpening Copy\n')
        copy = Nansat(ncfile)
        inc0 = orig['incidence_angle']
        inc1 = copy['incidence_angle']
        sys.stderr.write('\nGet orig grids\n')
        lon0, lat0 = orig.get_geolocation_grids()
        sys.stderr.write('\nGet copy grids\n')
        lon1, lat1 = copy.get_geolocation_grids()
        sys.stderr.write('\nGet orig sigma0_HH\n')
        sigma0_0 = orig['sigma0_HH']
        sys.stderr.write('\nGet copy sigma0_HH\n')
        sigma0_1 = copy['sigma0_HH']
        sys.stderr.write('\nAsserting\n')
        np.testing.assert_allclose(lon0, lon1)
        np.testing.assert_allclose(lat0, lat1)
        # If the next tests fail, it could indicate that the data is flipped
        # check by pyplot.imshow orig vs copy...
        np.testing.assert_allclose(inc0, inc1)
        np.testing.assert_allclose(sigma0_0, sigma0_1)
        os.unlink(ncfile)

    def incidence_angle(self, rsfile):
        sys.stderr.write('\nincidence_angle:'+rsfile+'\n')
        n = Nansat(rsfile)
        inc_min = float(n.get_metadata()['NEAR_RANGE_INCIDENCE_ANGLE'])-0.5
        inc_max = float(n.get_metadata()['FAR_RANGE_INCIDENCE_ANGLE'])+0.5
        inc = n['incidence_angle']
        assert np.all(np.greater_equal(inc[np.isnan(inc) == False], inc_min))
        assert np.all(np.less_equal(inc[np.isnan(inc) == False], inc_max))

    def export_band(self, rsfile):
        sys.stderr.write('\nexport_band:'+rsfile+'\n')
        orig = Nansat(rsfile)
        ncfile = 'test.nc'
        orig.export(ncfile, bands=[orig.get_band_number('incidence_angle')])
        copy = Nansat(ncfile)
        inc0 = orig['incidence_angle']
        inc1 = copy['incidence_angle']
        np.testing.assert_allclose(inc0, inc1)
        os.unlink(ncfile)

    def resize(self, rsfile):
        sys.stderr.write('\nresize:'+rsfile+'\n')
        n = Nansat(rsfile)
        inc_max = float(n.get_metadata()['FAR_RANGE_INCIDENCE_ANGLE'])+0.5
        n.resize(0.5, eResampleAlg=0)
        assert (np.nanmax(n['incidence_angle']) <= inc_max)
        n.undo()
        n.resize(0.5, eResampleAlg=1)
        assert (np.nanmax(n['incidence_angle']) <= inc_max)
        n.undo()
        n.resize(0.5, eResampleAlg=2)
        assert (np.nanmax(n['incidence_angle']) <= inc_max)
        n.undo()
        n.resize(0.5, eResampleAlg=3)
        assert (np.nanmax(n['incidence_angle']) <= inc_max)
        n.undo()
        n.resize(0.5, eResampleAlg=4)
        assert (np.nanmax(n['incidence_angle']) <= inc_max)
        n.undo()



import unittest
from nansat.mappers.mapper_opendap_ostia import Mapper


class MyWaveOpenDAPTests(unittest.TestCase):

    def setUp(self):
        self.src = 'https://podaac-opendap.jpl.nasa.gov/opendap/allData/ghrsst/data/L4/GLOB/' \
                   'UKMO/OSTIA/2016/006/20160106-UKMO-L4HRfnd-GLOB-v01-fv02-OSTIA.nc.bz2'

    def test_get_date(self):
        res = Mapper.get_date(self.src)
        self.assertIsInstance(res, str)
        self.assertEqual(res, '2016-01-06T00:00:00Z')


# Name: __init__.py
# Purpose: Use the current folder as a package
# Authors:      Asuka Yamakawa, Anton Korosov, Knut-Frode Dagestad,
#               Morten W. Hansen, Alexander Myasoyedov,
#               Dmitry Petrenko, Evgeny Morozov, Aleksander Vines
# Created:      29.06.2011
# Copyright:    (c) NERSC 2011 - 2015
# Licence:
# This file is part of NANSAT.
# NANSAT is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
# http://www.gnu.org/licenses/gpl-3.0.html
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
from __future__ import absolute_import
import os
import sys
import warnings
import importlib
pixfun_module_name = 'nansat._pixfun_py{0}'.format(sys.version_info[0])

# check if pixel functions were compiled using setup_tools
try:
    pixfun = importlib.import_module(pixfun_module_name)
    pixfun.registerPixelFunctions()
except ImportError as e:
    print(e)
    warnings.warn('''Cannot register C pixel functions!
                     Either nansat was not installed using setup.py or
                     pixel functions were not compiled automatically.
                     For development, use "python setup.py build_ext --inplace"
                     to compile pixel functions manually into the source tree.
                     ''')
from nansat.nsr import NSR
from nansat.domain import Domain
from nansat.nansat import Nansat
from nansat.figure import Figure

__all__ = ['NSR', 'Domain', 'Nansat', 'Figure']

os.environ['LOG_LEVEL'] = '30'



# Name:        mapper_ocean_prodcutivity
# Purpose:     Mapping for MODIS and SeaWiFS Level-3
#              from Ocean Productivity website (Oregon State University)
# Authors:      Dmitry Petrenko, Anton Korosov
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html

import datetime
import os.path
import glob

import numpy as np

from nansat.utils import gdal, ogr
from nansat.vrt import VRT

from nansat.exceptions import WrongMapperError

class Mapper(VRT):
    ''' Mapper for Ocean Productivity website
    http://www.science.oregonstate.edu/ocean.productivity/'''
    # detect wkv from metadata 'Parameter'
    param2wkv = {'chl': 'mass_concentration_of_chlorophyll_a_in_sea_water',
                 'sst': 'sea_surface_temperature',
                 'par': 'downwelling_photosynthetic_photon_radiance_in_sea_water',
                 'ipar': 'instantaneous_downwelling_photosynthetic_photon_radiance_in_sea_water',
                 'bbp': 'particle_backscatter_at_443_nm'
                 }

    bandNames = {'mass_concentration_of_chlorophyll_a_in_sea_water': 'algal_1',
                 'sea_surface_temperature': 'SST',
                 'instantaneous_downwelling_photosynthetic_photon_radiance_in_sea_water': 'ipar',
                 'particle_backscatter_at_443_nm': 'bbp_443'
                 }

    def __init__(self, filename, gdalDataset, gdalMetadata, **kwargs):
        ''' Ocean Productivity website VRT '''

        try:
            assert 'IDL' in gdalMetadata['Projection Category']
            assert '-9999' in gdalMetadata['Hole Value']
        except:
            raise WrongMapperError

        print('Ocean Productivity website data')
        # get list of similar (same date) files in the directory
        iDir, iFile = os.path.split(filename)
        iFileName, iFileExt = os.path.splitext(iFile)
        simFilesMask = os.path.join(iDir, '*' + iFileName[4:11] + iFileExt)
        #print 'simFilesMask', simFilesMask
        simFiles = glob.glob(simFilesMask)
        #print 'simFiles', simFiles

        metaDict = []
        for simFile in simFiles:
            #print 'simFile',simFile
            # open subdataset with GDAL
            tmpSourceFilename = simFile
            tmpGdalDataset = gdal.Open(tmpSourceFilename)

            # get metadata, get 'Parameter'
            tmpGdalMetadata = tmpGdalDataset.GetMetadata()
            iDir, ifilename = os.path.split(tmpSourceFilename)
            #print 'ifilename',ifilename
            simParameter = ifilename[0:3]

            # set params of the similar file
            simSourceFilename = tmpSourceFilename
            simGdalDataset = tmpGdalDataset
            simGdalMetadata = tmpGdalMetadata

            # get WKV from the similar file
            for param in self.param2wkv:
                #print 'param', param
                if param in simParameter:
                    simWKV = self.param2wkv[param]
                    break
            #print 'simWKV', simWKV
            # generate entry to metaDict
            metaEntry = {
                'src': {'SourceFilename': simSourceFilename,
                        'SourceBand':  1,
                        'ScaleRatio': float(simGdalMetadata['Slope']),
                        'ScaleOffset': float(simGdalMetadata['Intercept'])},
                'dst': {'wkv': simWKV,
                        'name': self.bandNames[simWKV],
                        'Parameter': simParameter}}
            #print 'metaEntry', metaEntry
            # append entry to metaDict
            metaDict.append(metaEntry)

        #get array with data and make 'mask'
        a = simGdalDataset.ReadAsArray()
        mask = np.zeros(a.shape, 'uint8') + 128
        mask[a < -9990] = 1
        self.band_vrts = {'maskVRT': VRT(array=mask)}

        metaDict.append({'src': {'SourceFilename': (self.band_vrts['maskVRT'].
                                                    filename),
                                 'SourceBand': 1},
                         'dst': {'name': 'mask'}})

        # create empty VRT dataset with geolocation only
        # print 'simGdalMetadata', simGdalMetadata
        latitudeStep = 0.08333334
        longitudeStep = 0.08333334
        numberOfColumns = 4320
        numberOfLines = 2160
        #longitudeStep = float(simGdalMetadata['Longitude Step'])
        self._init_from_dataset_params(numberOfColumns, numberOfLines,
                                       (-180.0, longitudeStep, 0.0, 90.0, 0.0, -longitudeStep),
                                       'GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]],AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0,AUTHORITY["EPSG","8901"]],UNIT["degree",0.01745329251994328,AUTHORITY["EPSG","9122"]],AUTHORITY["EPSG","4326"]]')

        # add bands with metadata and corresponding values to the empty VRT
        self.create_bands(metaDict)

        # Add valid time
        startYear = int(iFile[4:8])
        startDay = int(iFile[8:11])
        self.dataset.SetMetadataItem('time_coverage_start',
            (datetime.datetime(startYear, 1, 1) +
                       datetime.timedelta(startDay)).isoformat())



#------------------------------------------------------------------------------
# Name:         test_vrt.py
# Purpose:      Test the VRT class
#
# Author:       Anton Korosov
#
# Created:      15.01.2018
# Copyright:    (c) NERSC
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html
#------------------------------------------------------------------------------
import unittest
import os

import gdal
import numpy as np

from nansat.vrt import VRT
from nansat.geolocation import Geolocation
from nansat.tests import nansat_test_data as ntd

class GeolocationTest(unittest.TestCase):
    def setUp(self):
        self.test_file = os.path.join(ntd.test_data_path, 'gcps.tif')

    def test_init(self):
        lon, lat = np.meshgrid(np.linspace(0,5,10), np.linspace(10,20,30))
        x_vrt = VRT.from_array(lon)
        y_vrt = VRT.from_array(lat)

        ga = Geolocation(x_vrt, y_vrt)

        self.assertIsInstance(ga, Geolocation)
        self.assertEqual(ga.data['X_DATASET'], x_vrt.filename)
        self.assertEqual(ga.data['Y_DATASET'], y_vrt.filename)
        self.assertEqual(ga.data['LINE_OFFSET'], '0')
        self.assertEqual(ga.data['LINE_STEP'], '1')
        self.assertEqual(ga.data['PIXEL_OFFSET'], '0')
        self.assertEqual(ga.data['PIXEL_STEP'], '1')
        self.assertEqual(ga.data['SRS'], 'GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",'
                                         '6378137,298.257223563,AUTHORITY["EPSG","7030"]],AUT'
                                         'HORITY["EPSG","6326"]],PRIMEM["Greenwich",0,AUTHORI'
                                         'TY["EPSG","8901"]],UNIT["degree",0.0174532925199433'
                                         ',AUTHORITY["EPSG","9122"]],AUTHORITY["EPSG","4326"]]')
        self.assertEqual(ga.data['X_BAND'], '1')
        self.assertEqual(ga.data['Y_BAND'], '1')
        self.assertEqual(ga.x_vrt, x_vrt)
        self.assertEqual(ga.y_vrt, y_vrt)

    def test_from_dataset(self):
        ds = gdal.Open(self.test_file)
        g = Geolocation.from_dataset(ds)
        self.assertIsInstance(g, Geolocation)

    def test_from_filenames(self):
        lon, lat = np.meshgrid(np.linspace(0,5,10), np.linspace(10,20,30))
        x_vrt = VRT.from_array(lon)
        y_vrt = VRT.from_array(lat)
        g = Geolocation.from_filenames(x_vrt.filename, y_vrt.filename)
        self.assertIsInstance(g, Geolocation)
        self.assertEqual(g.data['X_DATASET'], x_vrt.filename)
        self.assertEqual(g.data['Y_DATASET'], y_vrt.filename)
        self.assertEqual(g.data['LINE_OFFSET'], '0')
        self.assertEqual(g.data['LINE_STEP'], '1')
        self.assertEqual(g.data['PIXEL_OFFSET'], '0')
        self.assertEqual(g.data['PIXEL_STEP'], '1')



# Name:         mapper_aapp_L1C.py
# Purpose:      Nansat mapper for AAPP Level 1C output
# Author:       Knut-Frode Dagestad
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html

# Description of file format:
# http://research.metoffice.gov.uk/research/interproj/nwpsaf/aapp/NWPSAF-MF-UD-003_Formats.pdf (page 120-)
import sys
import struct
import datetime
import warnings

from nansat.exceptions import WrongMapperError
from nansat.geolocation import Geolocation
from nansat.vrt import VRT

dataFormats = {1: 'LAC', 2: 'GAC', 3: 'HRPT'}
dataSetQualityIndicatorOffset = 114
recordLength = 29808
headerLength = recordLength
imageOffset = headerLength + 1092


class Mapper(VRT):
    ''' VRT with mapping of WKV for AVHRR L1C output from AAPP '''

    def __init__(self, filename, gdalDataset, gdalMetadata, **kwargs):

        ########################################
        # Read metadata from binary file
        ########################################
        try:
            fp = open(filename, 'rb')
        except IOError:
            raise WrongMapperError
        fp.seek(24)
        try:
            satID = int(struct.unpack('<l', fp.read(4))[0])
        except:
            raise WrongMapperError

        ##################
        # Read time
        ##################
        fp.seek(44)
        year = int(struct.unpack('<l', fp.read(4))[0])
        dayofyear = int(struct.unpack('<l', fp.read(4))[0])
        millisecondsOfDay = int(struct.unpack('<l', fp.read(4))[0])
        try:
            time = (datetime.datetime(year, 1, 1) +
                    datetime.timedelta(dayofyear - 1,
                                       milliseconds=millisecondsOfDay))
        except:
            raise WrongMapperError

        fp.seek(72)
        numScanLines = int(struct.unpack('<l', fp.read(4))[0])
        missingScanLines = int(struct.unpack('<l', fp.read(4))[0])
        numCalibratedScanLines = int(struct.unpack('<l', fp.read(4))[0])
        if missingScanLines != 0:
            print('WARNING: Missing scanlines: ' + str(missingScanLines))

        fp.seek(88)
        dataFormatNum = int(struct.unpack('<l', fp.read(4))[0])
        dataFormat = dataFormats[dataFormatNum]

        # Determine if we have channel 3A (daytime) or channel 3B (nighttime)
        def int2bitstring(s):
            return str(s) if s <= 1 else int2bitstring(s >> 1) + str(s & 1)
        fp.seek(headerLength + 20)
        scanlinebitFirstline = int(struct.unpack('<L', fp.read(4))[0])
        fp.seek(headerLength + recordLength*(numCalibratedScanLines-2) + 20)
        scanlinebitLastline = int(struct.unpack('<L', fp.read(4))[0])

        if int2bitstring(scanlinebitFirstline)[-1] == '0':
            startsWith3A = True
        else:
            startsWith3A = False
        if int2bitstring(scanlinebitLastline)[-1] == '0':
            endsWith3A = True
        else:
            endsWith3A = False

        if startsWith3A != endsWith3A:
            print('############################################')
            print('WARNING: channel 3 switches ')
            print('between daytime and nighttime (3A <-> 3B)')
            print('###########################################')

        ###########################
        # Make Geolocation Arrays
        ###########################
        srcRasterYSize = numCalibratedScanLines

        # Making VRT with raw (unscaled) lon and lat
        # (smaller bands than full dataset)
        self.band_vrts = {'RawGeolocVRT': VRT(srcRasterXSize=51,
                                            srcRasterYSize=srcRasterYSize)}
        RawGeolocMetaDict = []
        for lonlatNo in range(1, 3):
            RawGeolocMetaDict.append(
                {'src': {'SourceFilename': filename,
                         'SourceBand': 0,
                         'SourceType': "RawRasterBand",
                         'DataType': gdal.GDT_Int32,
                         'ImageOffset': (headerLength + 676 +
                                         (lonlatNo - 1) * 4),
                         'PixelOffset': 8,
                         'LineOffset': recordLength,
                         'ByteOrder': 'LSB'},
                 'dst': {}})

        self.band_vrts['RawGeolocVRT'].create_bands(RawGeolocMetaDict)

        # Make derived GeolocVRT with scaled lon and lat
        self.band_vrts['GeolocVRT'] = VRT(srcRasterXSize=51,
                                        srcRasterYSize=srcRasterYSize)
        GeolocMetaDict = []
        for lonlatNo in range(1, 3):
            GeolocMetaDict.append(
                {'src': {'SourceFilename': (self.band_vrts['RawGeolocVRT'].
                                            filename),
                         'SourceBand': lonlatNo,
                         'ScaleRatio': 0.0001,
                         'ScaleOffset': 0,
                         'DataType': gdal.GDT_Int32},
                 'dst': {}})

        self.band_vrts['GeolocVRT'].create_bands(GeolocMetaDict)

        GeolocObject = Geolocation(x_vrt=self.band_vrts['GeolocVRT'],
                                        y_vrt=self.band_vrts['GeolocVRT'],
                                        x_band=2, y_band=1,  # x = lon, y = lat
                                        line_offset=0, pixel_offset=25,
                                        line_step=1, pixel_step=40)

        #######################
        # Initialize dataset
        #######################
        # create empty VRT dataset with geolocation only
        # (from Geolocation Array)
        self._init_from_dataset_params(2048, numCalibratedScanLines,
                                        (0,1,0,numCalibratedScanLines,0,-1),
                                        GeolocObject.d['SRS'])
        self._add_geolocation(GeolocObject)


        ##################
        # Create bands
        ##################
        self.band_vrts['RawBandsVRT'] = VRT(
            srcRasterXSize=2048,
            srcRasterYSize=numCalibratedScanLines)
        RawMetaDict = []
        metaDict = []

        centralWavelengths = [0.63, 0.86, np.NaN, 10.8, 12.0]
        if startsWith3A:
            centralWavelengths[2] = 1.6
            firstIRband = 4
        else:
            centralWavelengths[2] = 3.7
            firstIRband = 3

        for bandNo in range(1, 6):
            RawMetaDict.append(
                {'src': {'SourceFilename': filename,
                         'SourceBand': 0,
                         'SourceType': "RawRasterBand",
                         'dataType': gdal.GDT_UInt16,
                         'ImageOffset': imageOffset + (bandNo - 1) * 2,
                         'PixelOffset': 10,
                         'LineOffset': recordLength,
                         'ByteOrder': 'LSB'},
                 'dst': {'dataType': gdal.GDT_UInt16}})

            if bandNo < firstIRband:
                wkv = 'albedo'
                minmax = '0 60'
            else:
                wkv = 'brightness_temperature'
                minmax = '290 210'

            metaDict.append(
                {'src': {'SourceFilename': (self.band_vrts['RawBandsVRT'].
                                            filename),
                         'SourceBand': bandNo,
                         'ScaleRatio': 0.01,
                         'ScaleOffset': 0,
                         'DataType': gdal.GDT_UInt16},
                 'dst': {'originalFilename': filename,
                         'dataType': gdal.GDT_Float32,
                         'wkv': wkv,
                         'colormap': 'gray',
                         'wavelength': centralWavelengths[bandNo-1],
                         'minmax': minmax}})

        # Add temperature difference between ch3 and ch 4 as pixelfunction
        if not startsWith3A:  # Only if ch3 is IR (nighttime)
            metaDict.append(
                {'src': [{'SourceFilename': (self.band_vrts['RawBandsVRT'].
                                             filename),
                          'ScaleRatio': 0.01,
                          'ScaleOffset': 0,
                          'SourceBand': 4},
                         {'SourceFilename': (self.band_vrts['RawBandsVRT'].
                                             filename),
                          'ScaleRatio': 0.01,
                          'ScaleOffset': 0,
                          'SourceBand': 3}],
                 'dst': {'PixelFunctionType': 'diff',
                         'originalFilename': filename,
                         'dataType': gdal.GDT_Float32,
                         'name': 'ch4-ch3',
                         'short_name': 'ch4-ch3',
                         'long_name': 'AVHRR ch4 - ch3 temperature difference',
                         'colormap': 'gray',
                         'units': 'kelvin',
                         'minmax': '-3 3'}})

        self.self.band_vrts['RawBandsVRT'].create_bands(RawMetaDict)
        self.create_bands(metaDict)

        globalMetadata = {}
        globalMetadata['satID'] = str(satID)
        globalMetadata['daytime'] = str(int(startsWith3A))
        self.dataset.SetMetadata(globalMetadata)

        # Adding valid time to dataset
        self.dataset.SetMetadataItem('time_coverage_start', time.isoformat())
        self.dataset.SetMetadataItem('time_coverage_end', time.isoformat())

        return



# Name:         mapper_landsat
# Purpose:      Mapping for LANDSAT*.tar.gz
# Authors:      Anton Korosov
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html
import os
import glob
import tarfile
import warnings
import datetime
import json

import pythesint as pti

from nansat.utils import gdal, np, parse_time
from nansat.vrt import VRT

from nansat.exceptions import WrongMapperError

class Mapper(VRT):
    ''' Mapper for LANDSAT5,6,7,8 .tar.gz or tif files'''

    def __init__(self, filename, gdalDataset, gdalMetadata,
                       resolution='low', **kwargs):
        ''' Create LANDSAT VRT from multiple tif files or single tar.gz file'''
        mtlFileName = ''
        bandFileNames = []
        bandSizes = []
        bandDatasets = []
        fname = os.path.split(filename)[1]

        if   (filename.endswith('.tar') or
              filename.endswith('.tar.gz') or
              filename.endswith('.tgz')):
            # try to open .tar or .tar.gz or .tgz file with tar
            try:
                tarFile = tarfile.open(filename)
            except:
                raise WrongMapperError

            # collect names of bands and corresponding sizes
            # into bandsInfo dict and bandSizes list
            tarNames = sorted(tarFile.getnames())
            for tarName in tarNames:
                # check if TIF files inside TAR qualify
                if   (tarName[0] in ['L', 'M'] and
                      os.path.splitext(tarName)[1] in ['.TIF', '.tif']):
                    # open TIF file from TAR using VSI
                    sourceFilename = '/vsitar/%s/%s' % (filename, tarName)
                    gdalDatasetTmp = gdal.Open(sourceFilename)
                    # keep name, GDALDataset and size
                    bandFileNames.append(sourceFilename)
                    bandSizes.append(gdalDatasetTmp.RasterXSize)
                    bandDatasets.append(gdalDatasetTmp)
                elif (tarName.endswith('MTL.txt') or
                      tarName.endswith('MTL.TXT')):
                    # get mtl file
                    mtlFileName = tarName

        elif ((fname.startswith('L') or fname.startswith('M')) and
              (fname.endswith('.tif') or
               fname.endswith('.TIF') or
               fname.endswith('._MTL.txt'))):

            # try to find TIF/tif files with the same name as input file
            path, coreName = os.path.split(filename)
            coreName = os.path.splitext(coreName)[0].split('_')[0]
            coreNameMask = coreName+'*[tT][iI][fF]'
            tifNames = sorted(glob.glob(os.path.join(path, coreNameMask)))
            for tifName in tifNames:
                sourceFilename = tifName
                gdalDatasetTmp = gdal.Open(sourceFilename)
                # keep name, GDALDataset and size
                bandFileNames.append(sourceFilename)
                bandSizes.append(gdalDatasetTmp.RasterXSize)
                bandDatasets.append(gdalDatasetTmp)

            # get mtl file
            mtlFiles = glob.glob(coreName+'*[mM][tT][lL].[tT][xX][tT]')
            if len(mtlFiles) > 0:
                mtlFileName = mtlFiles[0]
        else:
            raise WrongMapperError

        # if not TIF files found - not appropriate mapper
        if not bandFileNames:
            raise WrongMapperError

        # get appropriate band size based on number of unique size and
        # required resoltuion
        if resolution == 'low':
            bandXSise = min(bandSizes)
        elif resolution in ['high', 'hi']:
            bandXSise = max(bandSizes)
        else:
            raise ValueError('Wrong resolution %s for file %s' % (resolution, filename))

        # find bands with appropriate size and put to metaDict
        metaDict = []
        for bandFileName, bandSize, bandDataset in zip(bandFileNames,
                                                       bandSizes,
                                                       bandDatasets):
            if bandSize == bandXSise:
                # let last part of file name be suffix
                bandSuffix = os.path.splitext(bandFileName)[0].split('_')[-1]

                metaDict.append({
                    'src': {'SourceFilename': bandFileName,
                            'SourceBand':  1,
                            'ScaleRatio': 0.1},
                    'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                            'suffix': bandSuffix}})
                gdalDataset4Use = bandDataset

        # create empty VRT dataset with geolocation only
        self._init_from_gdal_dataset(gdalDataset4Use)

        # add bands with metadata and corresponding values to the empty VRT
        self.create_bands(metaDict)

        if len(mtlFileName) > 0:
            mtlFileName = os.path.join(os.path.split(bandFileNames[0])[0],
                                        mtlFileName)
            mtlFileLines = [line.strip() for line in self.read_vsi(mtlFileName).split('\n')]
            dateString = [line.split('=')[1].strip()
                          for line in mtlFileLines
                            if ('DATE_ACQUIRED' in line or
                              'ACQUISITION_DATE' in line)][0]
            timeStr = [line.split('=')[1].strip()
                        for line in mtlFileLines
                            if ('SCENE_CENTER_TIME' in line or
                                'SCENE_CENTER_SCAN_TIME' in line)][0]
            time_start = parse_time(dateString + 'T' + timeStr).isoformat()
            time_end = (parse_time(dateString + 'T' + timeStr) +
                        datetime.timedelta(microseconds=60000000)).isoformat()

        self.dataset.SetMetadataItem('time_coverage_start', time_start)
        self.dataset.SetMetadataItem('time_coverage_end', time_end)

        # set platform
        platform = 'LANDSAT'
        if fname[2].isdigit():
            platform += '-'+fname[2]
        ee = pti.get_gcmd_platform(platform)
        self.dataset.SetMetadataItem('platform', json.dumps(ee))

        # set instrument
        instrument = {
        'LANDSAT' : 'MSS',
        'LANDSAT-1' : 'MSS',
        'LANDSAT-2' : 'MSS',
        'LANDSAT-3' : 'MSS',
        'LANDSAT-4' : 'TM',
        'LANDSAT-5' : 'TM',
        'LANDSAT-7' : 'ETM+',
        'LANDSAT-8' : 'OLI'}[platform]
        ee = pti.get_gcmd_instrument(instrument)
        self.dataset.SetMetadataItem('instrument', json.dumps(ee))




# Name:    utils.py
# Purpose: collection of data and funcs used in NANSAT modules
# Authors:      Asuka Yamakawa, Anton Korosov, Knut-Frode Dagestad,
#               Morten W. Hansen, Alexander Myasoyedov,
#               Dmitry Petrenko, Evgeny Morozov
# Created:      29.06.2011
# Copyright:    (c) NERSC 2011 - 2013
# Licence:
# This file is part of NANSAT.
# NANSAT is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
# http://www.gnu.org/licenses/gpl-3.0.html
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
from __future__ import absolute_import

import os
import warnings
import logging
from dateutil.parser import parse

try:
    import matplotlib
    if 'DISPLAY' not in os.environ:
        matplotlib.use('Agg')
    from matplotlib.colors import hex2color
    from matplotlib import cm
except ImportError:
    MATPLOTLIB_IS_INSTALLED = False
else:
    MATPLOTLIB_IS_INSTALLED = True

import numpy as np

try:
    import gdal, ogr, osr
except:
    from osgeo import gdal, ogr, osr
gdal.UseExceptions()

NUMPY_TO_GDAL_TYPE_MAP = {
        'uint8': 1,
        'int8': 1,
        'uint16': 2,
        'int16': 3,
        'uint32': 4,
        'int32': 5,
        'float32': 6,
        'float64': 7,
        'complex64': 10,
        'complex128': 11
    }

def remove_keys(dict, keys):
    if keys is None:
        keys = []
    for key in keys:
        dict.pop(key, None)
    return dict

def register_colormaps():
    ''' Create custom colormaps and register them '''
    obpg = {'red': [(0.00, 0.56, 0.56),
                    (0.19, 0.00, 0.00),
                    (0.38, 0.00, 0.00),
                    (0.50, 0.00, 0.00),
                    (0.63, 1.00, 1.00),
                    (0.88, 1.00, 1.00),
                    (1.00, 0.40, 0.40)],

            'green': [(0.00, 0.00, 0.00),
                      (0.19, 0.00, 0.00),
                      (0.38, 1.00, 1.00),
                      (0.50, 1.00, 1.00),
                      (0.63, 1.00, 1.00),
                      (0.88, 0.00, 0.00),
                      (1.00, 0.00, 0.00)],

            'blue': [(0.00, 0.43, 0.43),
                     (0.19, 1.00, 1.00),
                     (0.38, 1.00, 1.00),
                     (0.50, 0.00, 0.00),
                     (0.63, 0.00, 0.00),
                     (0.88, 0.00, 0.00),
                     (1.00, 0.00, 0.00)],
            }


    ak01 = {'red': [(0, 0.1, 0.1,),
                    (0.1, 0.56, 0.56,),
                    (0.22, 0, 0,),
                    (0.27, 0, 0,),
                    (0.37, 0.3, 0.3,),
                    (0.47, 0, 0,),
                    (0.52, 0, 0,),
                    (0.64, 1, 1,),
                    (0.76, 1, 1,),
                    (0.88, 0.4, 0.4,),
                    (1, 1, 1,)],

            'green': [(0, 0, 0,),
                      (0.1, 0, 0,),
                      (0.22, 0, 0,),
                      (0.27, 0, 0,),
                      (0.37, 0.6, 0.6,),
                      (0.47, 0.6, 0.6,),
                      (0.52, 1, 1,),
                      (0.64, 1, 1,),
                      (0.76, 0, 0,),
                      (0.88, 0, 0,),
                      (1, 0.5, 0.5,)],

            'blue': [(0, 0.1, 0.1,),
                     (0.1, 0.5, 0.5,),
                     (0.22, 0.5, 0.5,),
                     (0.27, 1, 1,),
                     (0.37, 1, 1,),
                     (0.47, 0, 0,),
                     (0.52, 0, 0,),
                     (0.64, 0, 0,),
                     (0.76, 0, 0,),
                     (0.88, 0, 0,),
                     (1, 0.5, 0.5,)],
            }

    if MATPLOTLIB_IS_INSTALLED:
        cm.register_cmap(name='obpg', data=obpg, lut=256)
        cm.register_cmap(name='ak01', data=ak01, lut=256)

def initial_bearing(lon1, lat1, lon2, lat2):
        """Initial bearing when traversing from point1 (lon1, lat1)
        to point2 (lon2, lat2)

        See http://www.movable-type.co.uk/scripts/latlong.html

        Parameters
        ----------
        lon1, lat1 : float
            longitude and latitude of start point
        lon2, lat2 : float
            longitude and latitude of end point

        Returns
        -------
        initial_bearing : float
            The initial bearing (azimuth direction) when heading out
            from the start point towards the end point along a great circle.

        """
        rlon1 = np.radians(lon1)
        rlat1 = np.radians(lat1)
        rlon2 = np.radians(lon2)
        rlat2 = np.radians(lat2)
        bearing = np.arctan2(np.sin(rlon2 - rlon1) * np.cos(rlat2),
                             np.cos(rlat1) * np.sin(rlat2) -
                             np.sin(rlat1) * np.cos(rlat2) *
                             np.cos(rlon2 - rlon1))
        return np.mod(np.degrees(bearing) + 360, 360)


def haversine(lon1, lat1, lon2, lat2):
    """
    Calculate the great circle distance between two points
    on the spherical earth (specified in decimal degrees)
    """
    # convert decimal degrees to radians
    lon1, lat1, lon2, lat2 = map(np.radians, [lon1, lat1, lon2, lat2])
    # haversine formula
    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = np.sin(dlat/2)**2 + np.cos(lat1) * np.cos(lat2) * np.sin(dlon/2)**2
    c = 2 * np.arcsin(np.sqrt(a))
    distance_meters = 6367000 * c
    return distance_meters


def add_logger(logName='', logLevel=None):
    """ Creates and returns logger with default formatting for Nansat

    Parameters
    -----------
    logName : string, optional
        Name of the logger

    Returns
    --------
    logging.logger

    See also
    --------
    `<http://docs.python.org/howto/logging.html>`_

    """
    if logLevel is not None:
        os.environ['LOG_LEVEL'] = str(logLevel)
    # create (or take already existing) logger
    # with default logging level WARNING
    logger = logging.getLogger(logName)
    logger.setLevel(int(os.environ['LOG_LEVEL']))

    # if logger already exits, default stream handler has been already added
    # otherwise create and add a new handler
    if len(logger.handlers) == 0:
        # create console handler and set level to debug
        ch = logging.StreamHandler()
        # create formatter
        formatter = logging.Formatter('%(asctime)s|%(levelno)s|%(module)s|'
                                      '%(funcName)s|%(message)s',
                                      datefmt='%I:%M:%S')
        # add formatter to ch
        ch.setFormatter(formatter)
        # add ch to logger
        logger.addHandler(ch)

    logger.handlers[0].setLevel(int(os.environ['LOG_LEVEL']))

    return logger


def get_random_color(c0=None, minDist=100, low=0, high=255):
    """Create random color which is far enough from the input color

    Parameters
    ----------
        c0 : str
            hexademical representation of the color (e.g. '#ff0000' for red)
        minDist : int
            minimal distance to input color

    Returns
    -------
        c0 : str
            hexademical representation of the new random color
    """
    if not MATPLOTLIB_IS_INSTALLED:
        raise ImportError('Matplotlib is not installed')

    # check inputs
    if c0 is None:
        c0 = '#000000'
    # convert input color to tuple of R,G,B
    c0rgb = np.array(hex2color(c0))

    # create new random color
    c1rgb = np.array([np.random.randint(low, high),
                      np.random.randint(low, high),
                      np.random.randint(low, high)])

    # calculate distance
    d = np.sum((c0rgb - c1rgb)**2)**0.5

    # if distance is small, create new random color
    if d < minDist:
        c1 = get_random_color(c0, minDist)
    else:
        # convert to HEX code
        c1 = '#%02x%02x%02x' % tuple(c1rgb)

    return c1


def parse_time(time_string):
    ''' Parse time string accounting for possible wrong formatting
    Parameters
    ----------
    time_string : str
        string with date and time
    Returns
    -------
        time_value : datetime object

    '''
    time_string = time_string.strip()
    # To account for datasets on the format YYYY-MM-DDZ which is
    # invalid since it has no time, but a timezone
    try:
        time_value = parse(time_string)
    except ValueError:
        if (len(time_string) == 11 and
                time_string.endswith('Z')):
            time_value = parse(time_string[:10])

    return time_value

register_colormaps()

numpy_to_gdal_type = {
    'uint8': 'Byte',
    'int8': 'Byte',
    'uint16': 'UInt16',
    'int16': 'Int16',
    'uint32': 'UInt32',
    'int32': 'Int32',
    'float32': 'Float32',
    'float64': 'Float64',
    'complex64': 'CFloat32',
    'complex128': 'CFloat64'}

gdal_type_to_offset = {
    'Byte': '1',
    'UInt16': '2',
    'Int16': '2',
    'UInt32': '4',
    'Int32': '4',
    'Float32': '4',
    'Float64': '8',
    'CFloat32': '8',
    'CFloat64': '16'}



# coding=utf-8
#------------------------------------------------------------------------------
# Name:         test_mapper_opendap.py
# Purpose:      Test the Opendap class
#
# Author:       Artem Moiseev
#
# Created:      15.08.2018
# Copyright:    (c) NERSC
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html
#------------------------------------------------------------------------------
import sys
import unittest
from netCDF4 import Dataset
import tempfile
from mock import patch
from nansat.exceptions import WrongMapperError
from collections import OrderedDict
from nansat.mappers.opendap import Opendap
import numpy as np
import warnings


class OpenDAPTests(unittest.TestCase):

    def setUp(self):
        _, tmp_filename = tempfile.mkstemp(suffix='.nc')
        ds = Dataset(tmp_filename, 'w')
        lat_sz = 30
        lon_sz = 20
        values = np.random.random_sample((lat_sz, lon_sz))

        # Set dimensions
        ds.createDimension('lat', lat_sz)
        ds.createDimension('lon', lon_sz)
        ds.createDimension('time', 3)
        ds.createDimension('depth', 10)
        # Set variables
        # 1d "dimensional" variables i.e lats, times, etc.
        times = ds.createVariable('var2', 'i4', ('time'))
        lats = ds.createVariable('lat', 'i4', ('lat'))
        lats[:] = np.linspace(0, 60, lat_sz)
        lons = ds.createVariable('lon', 'i4', ('lon'))
        lons[:] = np.linspace(0, 20, lon_sz)
        # Spatial variables 2d, 3d, and 4d
        ds.createVariable('var2d', 'i4', ('lat', 'lon'))
        ds.createVariable('var3d', 'i4', ('time', 'lat', 'lon'))
        ds.createVariable('var4d', 'f4', ('time', 'depth', 'lat', 'lon'))
        # Initiate the Opendap object
        self.od = Opendap()
        self.od.baseURLs = ['http://first.no', 'http://second.com']
        self.od.xName = 'lon'
        self.od.yName = 'lat'
        self.od.timeVarName = 'time'
        self.od.ds = ds
        self.ds = ds

    def test_test_mapper(self):
        res_ok = self.od.test_mapper(filename='http://first.no/path/to/the/file.nc')
        self.assertIsNone(res_ok)
        with self.assertRaises(WrongMapperError):
            self.od.test_mapper(filename='http://not-in-base-urls.net/path/to/the/file.nc')

    """ Not sure how to test get_dataset.. 
    
    According toe the followingm, mocking C modules seems not to be possible.. Can that be the
    reason the below doesn't work?

    See https://stackoverflow.com/questions/192649/can-you-monkey-patch-methods-on-core-types-in-python/192857#192857
    """
    ##@patch('nansat.mappers.opendap.Dataset.__init__')
    #@patch.object('nansat.mappers.opendap.Dataset', '__init__', return_value = None)
    #def test_get_dataset(self, mock_dataset):
    #    dd = Dataset()
    #    self.od.filename = 'http://something.that.is/mocked'
    #    ds1 = self.od.get_dataset(None)
    #    self.assertIsInstance(ds1, Dataset)

    def test_get_dataset_which_does_not_exist(self):
        wrong_filename = '/path/which/does/not/exist/file.nc'
        self.od.filename = wrong_filename
        if sys.version_info.major==2:
            with self.assertRaises(IOError) as ve:
                self.od.get_dataset(None)
        else:
            with self.assertRaises(FileNotFoundError) as ve:
                self.od.get_dataset(None)
        self.assertEqual(ve.exception.errno, 2)

        with self.assertRaises(ValueError) as ve:
            self.od.get_dataset([])
        self.assertEqual(ve.exception.args[0], 'Input ds is not netCDF.Dataset!')

    def test_get_geospatial_variable_names(self):
        ds_vars = self.od.get_geospatial_variable_names()
        self.assertEqual(len(ds_vars), 3)
        self.assertIn('var2d', ds_vars)
        self.assertIn('var3d', ds_vars)
        self.assertIn('var4d', ds_vars)

    def test_get_dataset_time(self):
        pass

    def test_get_layer_datetime(self):
        date1 = '2010-01-02'
        datetimes_1 = np.arange(np.datetime64('2010-01-01'), np.datetime64('2010-01-02'))
        res_layer_num1, res_layer_date1 = Opendap.get_layer_datetime(date1, datetimes_1)

        self.assertEqual(res_layer_num1, 0)
        self.assertIsInstance(res_layer_date1, np.datetime64)

        datetimes_2 = np.arange(np.datetime64('2009-01-01'), np.datetime64('2009-01-08'))

        with self.assertRaises(ValueError):
            Opendap.get_layer_datetime(date1, datetimes_2)

        datetimes_3 = np.arange(np.datetime64('2009-12-31'), np.datetime64('2010-01-08'))
        res_layer_num3, res_layer_date3 = Opendap.get_layer_datetime(date1, datetimes_3)
        self.assertEqual(res_layer_date3, np.datetime64(date1))
        self.assertEqual(res_layer_num3, 2)

    @patch('nansat.mappers.opendap.gdal.Open')
    def test_metaitem(self, mock_open):
        mock_open.return_value = None
        self.od.ds.variables['var3d'].setncattr('test_attr', 'test_val')
        res1 = self.od.get_metaitem('https://file-url.nc', 'var3d', (0, 'y', 'x'))
        self.assertIsInstance(res1, dict)
        self.assertEqual(len(res1.keys()), 2)
        self.assertIn('dst', res1.keys())
        self.assertIn('src', res1.keys())
        self.assertIn('SourceFilename', res1['src'].keys())
        self.assertIn('SourceBand', res1['src'].keys())
        self.assertIn('name', res1['dst'].keys())
        self.assertIn('dataType', res1['dst'].keys())
        self.assertIn('test_attr', res1['dst'].keys())
        self.od.ds.variables['var3d'].delncattr('test_attr')

    @patch('nansat.mappers.opendap.gdal.Open')
    def test_get_metaitem_spec_attrs(self, mock_open):
        mock_open.return_value = None
        test_cases = [
            {'key': 'offset', 'val': 'offset_value', 'meta_key': 'ScaleOffset'},
            {'key': 'add_offset', 'val': 'add_offset_value', 'meta_key': 'ScaleOffset'},
            {'key': 'scale', 'val': 'scale_value', 'meta_key': 'ScaleRatio'},
            {'key': 'scale_factor', 'val': 'scale_factor_value', 'meta_key': 'ScaleRatio'},
        ]

        for test_case in test_cases:
            self.od.ds.variables['var3d'].setncattr(test_case['key'], test_case['val'])
            res1 = self.od.get_metaitem('https://file-url.nc', 'var3d', (0, 'y', 'x'))
            self.assertIn((test_case['meta_key'], test_case['val']), res1['src'].items())
            self.od.ds.variables['var3d'].delncattr(test_case['key'])

    def test_fix_encoding(self):
        self.assertEqual(Opendap._fix_encoding(u'åsnes'), 'snes')
        self.assertEqual(Opendap._fix_encoding('asnes'), 'asnes')

    def test_filter_dimensions(self):
        res1 = list(filter(self.od._filter_dimensions, ['time', 'var1', 'lat', 'lon']))
        self.assertIsInstance(res1, list)
        self.assertEqual(len(res1), 1)
        self.assertEqual(res1[0], 'var1')
        res2 = list(filter(self.od._filter_dimensions, ['time', 'lat', 'lon']))
        self.assertEqual(len(res2), 0)

    @patch('nansat.mappers.opendap.gdal.Open')
    def test_create_metadict(self, mock_open):
        mock_open.return_value = None
        res1 = self.od.create_metadict('test.nc', ['var4d'], 0)
        self.assertIsInstance(res1, list)
        self.assertEqual(len(res1), 10)

        for i in range(len(res1)):
            source_filename = res1[i]['src']['SourceFilename'][19:31]
            self.assertEqual(source_filename, '[0][%s][y][x]' % i)

        res2 = self.od.create_metadict('test.nc', ['var2d', 'var3d'], 0)
        self.assertEqual(len(res2), 2)
        self.assertEqual(res2[0]['src']['SourceFilename'][19:25], '[y][x]')
        self.assertEqual(res2[1]['src']['SourceFilename'][19:28], '[0][y][x]')

    def test_get_time_coverage_resolution(self):
        res1 = self.od.get_time_coverage_resolution()
        self.assertIsInstance(res1, int)
        self.assertEqual(res1, 0)
        self.od.ds.setncattr('time_coverage_resolution', 'P2D')
        res2 = self.od.get_time_coverage_resolution()
        self.assertEqual(res2, 172800)
        self.od.ds.setncattr('time_coverage_resolution', 'wrong_value')
        with warnings.catch_warnings():
            _ = self.od.get_time_coverage_resolution()
        self.od.ds.delncattr('time_coverage_resolution')

    def test_get_shape(self):
        res = self.od.get_shape()
        self.assertIsInstance(res, tuple)
        self.assertEqual(len(res), 2)
        self.assertEqual(res[0], 20)
        self.assertEqual(res[1], 30)

    def test_get_geotransfort(self):
        res = self.od.get_geotransform()
        self.assertIsInstance(res, tuple)
        self.assertEqual(len(res), 6)
        self.assertEqual(res, (0, 1, 0, 0, 0, 2))



# ------------------------------------------------------------------------------
# Name:         mapper_test_archive.py
# Purpose:      To discover test data for the integration tests
#
# Author:       Anton Korosov, Morten Wergeland Hansen, Asuka Yamakawa
# Modified:     Morten Wergeland Hansen, Aleksander Vines
#
# Created:      2014-06-18
# Last modified:2015-12-28 13:36
# Copyright:    (c) NERSC
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html
# ------------------------------------------------------------------------------
import os
import glob
import warnings

class DataForTestingMappers(object):
    def __init__(self):
        ''' Find test files and corresponding mapper names '''
        existingTestFiles = self.find_existing_files()
        self.mapperData = self.identify_mappers(existingTestFiles)

    def find_existing_files(self):
        ''' Find all files for testing inside MAPPER_TEST_DATA_DIR'''
        testFiles = []

        testDataEnv = os.getenv('MAPPER_TEST_DATA_DIR')
        if testDataEnv is None:
            warnings.warn('MAPPER_TEST_DATA_DIR is not defined')
        else:
            testDataDirs = testDataEnv.split(':')
            for testDataDir in testDataDirs:
                if os.path.isdir(testDataDir):
                    testFiles += glob.glob(os.path.join(testDataDir, '*', '*'))

        testFiles = [f for f in testFiles if self.readable(f)]

        return testFiles

    def identify_mappers(self, testFiles):
        ''' Get the name of the mapper from the sub-directory name '''

        return [{'fileName' : testFile,
                 'mapperName' : os.path.split(os.path.split(testFile)[0])[1]}
                for testFile in testFiles]

    def readable(self, testFile):
        ''' Test if file is readable at OS level '''
        if not os.path.exists(testFile):
            return False
        if not os.access(testFile, os.R_OK):
            return False
        if os.stat(testFile).st_size == 0:
            return False
        if os.path.isdir(testFile):
            return False

        return True

class DataForTestingOnlineMappers(object):
    mapperData = [
        {
        'fileName' : 'http://dap.ceda.ac.uk/data/neodc/esacci/sst/data/lt/Analysis/L4/v01.1/2010/05/01/20100501120000-ESACCI-L4_GHRSST-SSTdepth-OSTIA-GLOB_LT-v02.0-fv01.1.nc',
        'mapperName': 'opendap_sstcci',
        'bands': ['analysed_sst'],

        },{
        'fileName' : 'https://rsg.pml.ac.uk/thredds/dodsC/CCI_ALL-v2.0-8DAY',
        'mapperName' : 'opendap_occci',
        'date' : '2010-01-01',
        'bands': ['chlor_a'],
        },{
        'fileName' : 'https://rsg.pml.ac.uk/thredds/dodsC/CCI_ALL-v2.0-MONTHLY',
        'mapperName' : 'opendap_occci',
        'date' : '2010-01-01',
        'bands': ['chlor_a'],


        },{
        'fileName' : 'http://www.ifremer.fr/opendap/cerdap1/globcurrent/v2.0/global_025_deg/total_hs/2010/002/20100102000000-GLOBCURRENT-L4-CUReul_hs-ALT_SUM-v02.0-fv01.0.nc',
        'mapperName' : 'opendap_globcurrent',
        'bands': ['eastward_eulerian_current_velocity'],

        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/cryoclim/met.no/osisaf-nh/osisaf-nh_aggregated_ice_concentration_nh_polstere-100_197810010000.nc',
        'mapperName' : 'opendap_osisaf',
        'bands': ['ice_conc_avg'],
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/cryoclim/met.no/osisaf-nh-agg',
        'mapperName' : 'opendap_osisaf',
        'bands': ['ice_conc_avg'],
        'date': '1980-07-22',
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/osisaf/met.no/ice/conc/2016/04/ice_conc_sh_polstere-100_multi_201604261200.nc',
        'mapperName' : 'opendap_osisaf',
        'bands': ['ice_conc'],
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/osisaf/met.no/ice/drift_lr/merged/2016/04/ice_drift_nh_polstere-625_multi-oi_201604151200-201604171200.nc',
        'mapperName' : 'opendap_osisaf',
        'bands': ['dX', 'dY'],
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/osisaf/met.no/ice/type/2016/04/ice_type_nh_polstere-100_multi_201604151200.nc',
        'mapperName' : 'opendap_osisaf',
        'bands': ['ice_type'],
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/osisaf/met.no/ice/edge/2016/04/ice_edge_nh_polstere-100_multi_201604241200.nc',
        'mapperName' : 'opendap_osisaf',
        'bands': ['ice_edge'],

        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/myocean/siw-tac/sst-metno-arc-sst03/20121001000000-METNO-L4_GHRSST-SSTfnd-METNO_OI-ARC-v02.0-fv01.0.nc',
        'mapperName' : 'opendap_siwtacsst',
        'bands': ['analysed_sst'],
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/myocean/siw-tac/sst-metno-arc-sst03_V1/20120808-METNO-L4UHfnd-ARC-v01-fv01-METNO_OI.nc',
        'mapperName' : 'opendap_siwtacsst',
        'bands': ['analysed_sst'],
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/sea_ice/SST-METNO-ARC-SST_L4-OBS-V2-V1/sst_arctic_aggregated',
        'mapperName' : 'opendap_siwtacsst',
        'bands': ['analysed_sst'],
        'date': '2012-08-08'
        },{
        'fileName' : 'https://thredds.met.no/thredds/dodsC/aromearcticarchive/2017/10/30/arome_arctic_full_2_5km_20171030T21Z.nc',
        'mapperName' : 'opendap_arome',
        'bands': ['x_wind_10m', 'y_wind_10m'],
        'date': '2017-10-30'
        },{'fileName' : 'http://thredds.met.no/thredds/dodsC/meps25epsarchive/2017/10/26/meps_mbr1_pp_2_5km_20171026T06Z.nc',
        'mapperName' : 'opendap_arome',
        'bands': ['x_wind_10m', 'y_wind_10m'],
        'date': '2017-10-26T07:00'
        },{
        'fileName' : 'https://podaac-opendap.jpl.nasa.gov/opendap/allData/ghrsst/data/L4/GLOB/UKMO/OSTIA/2016/006/20160106-UKMO-L4HRfnd-GLOB-v01-fv02-OSTIA.nc.bz2',
        'mapperName' : 'opendap_ostia',
        'bands': ['analysed_sst', 'mask'],
        'date': '2016-01-06'
        },{
        'fileName' : 'http://thredds.met.no/thredds/dodsC/fou-hi/mywavewam4archive/2017/10/29/MyWave_wam4_WAVE_20171029T18Z.nc',
        'mapperName' : 'opendap_mywave4km',
        'bands': ['hs'],
        'date': '2017-10-29T18:00Z'
        }
        ]



import re
import json
import numpy as np
from datetime import datetime
import pythesint as pti

from netCDF4 import Dataset

import gdal

from nansat.vrt import VRT
from nansat.geolocation import Geolocation
from nansat.nsr import NSR
from nansat.domain import Domain
from nansat.mappers.scatterometers import Mapper as ScatterometryMapper
from nansat.exceptions import WrongMapperError

class Mapper(ScatterometryMapper):
    """ Nansat mapper for ASCAT """

    def __init__(self, filename, gdal_dataset, metadata, quartile=0, *args, **kwargs):

        if not 'ascat' in metadata.get('NC_GLOBAL#source', '').lower():
            raise WrongMapperError

        super(Mapper, self).__init__(filename, gdal_dataset, metadata, quartile=quartile, *args, **kwargs)

        lat = self.dataset.GetRasterBand(self._latitude_band_number(gdal_dataset)).ReadAsArray()
        lon = self.dataset.GetRasterBand(self._longitude_band_number(gdal_dataset)).ReadAsArray()
        lon = ScatterometryMapper.shift_longitudes(lon)
        self.set_gcps(lon, lat, gdal_dataset)

        # Get dictionary describing the instrument and platform according to
        # the GCMD keywords
        ii = pti.get_gcmd_instrument('ascat')
        pp = pti.get_gcmd_platform(metadata['NC_GLOBAL#source'].split(' ')[0])
        provider = pti.get_gcmd_provider(re.split('[^a-zA-Z]',
            metadata['NC_GLOBAL#institution'])[0])

        # TODO: Validate that the found instrument and platform are indeed what
        # we want....

        self.dataset.SetMetadataItem('instrument', json.dumps(ii))
        self.dataset.SetMetadataItem('platform', json.dumps(pp))
        self.dataset.SetMetadataItem('data_center', json.dumps(provider))
        self.dataset.SetMetadataItem('entry_title', metadata['NC_GLOBAL#title'])
        self.dataset.SetMetadataItem('ISO_topic_category',
                json.dumps(pti.get_iso19115_topic_category('Oceans')))

    def times(self):
        """ Get times from time variable
        """
        ds = Dataset(self.input_filename)

        # Get datetime object of epoch and time_units string
        time_units = self._time_reference(ds=ds)

        # Get all times - slight difference from NetCDF-CF mappers times method...
        times = ds.variables[self._timevarname(ds=ds)][:,0]

        # Create numpy array of np.datetime64 times (provide epoch to save time)
        tt = np.array([self._time_count_to_np_datetime64(tn,
            time_units=time_units) for tn in times])

        return tt



import unittest
from nansat.mappers.mapper_opendap_mywave import Mapper


class MyWaveOpenDAPTests(unittest.TestCase):

    def setUp(self):
        self.src = 'http://thredds.met.no/thredds/dodsC/fou-hi/mywavewam4archive' \
                   '/2017/10/29/MyWave_wam4_WAVE_20171029T18Z.nc'

    def test_get_date(self):
        res = Mapper.get_date(self.src)
        self.assertIsInstance(res, str)
        self.assertEqual(res, '2017-10-29T18:00:00Z')



# Name:         warnings.py
# Purpose:      Definitions of nansat warnings
# Authors:      Morten W. Hansen
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html

class NansatFutureWarning(Warning):
    pass



# Name:         mapper_metno_hires_seaice.py
# Purpose:      Nansat mapping for high resolution sea ice
#               from met.no Thredds server
# Authors:      Knut-Frode Dagestad
# Licence:      This file is part of NANSAT. You can redistribute it or modify
#               under the terms of GNU General Public License, v.3
#               http://www.gnu.org/licenses/gpl-3.0.html

# High resolution (1 km) manual ice concentration, based on SAR imagery
#
# Mapper is called with keyword (fake filename):
#   'metno_hires_seaice_20140109'
# and [optional] keyword: iceFolder = <local folder
# where netCDF files with ice are stored>
#
# The closest available data within +/- 3 days is returned
from __future__ import print_function, absolute_import, unicode_literals
import sys
import os
from datetime import datetime, timedelta

from nansat.utils import gdal, ogr
from nansat.exceptions import WrongMapperError
from nansat.vrt import VRT
import nansat.mappers.mapper_generic as mg


class Mapper(mg.Mapper):
    """Create VRT with mapping of WKV for Met.no seaice"""

    def __init__(self, filename, gdalDataset, gdalMetadata, **kwargs):
        """Create VRT"""

        try:
            ice_folder_name = kwargs['iceFolder']
        except:
            #iceFolderName = '/vol/istjenesten/data/metnoCharts/'
            ice_folder_name = '/vol/data/metnoCharts/'

        keyword_base = 'metno_local_hires_seaice'

        if filename[0:len(keyword_base)] != keyword_base:
            raise WrongMapperError

        keyword_time = filename[len(keyword_base)+1:]
        requested_time = datetime.strptime(keyword_time, '%Y%m%d')
        # Search for nearest available file, within the closest 3 days
        found_dataset = False
        for delta_day in [0, -1, 1, -2, 2, -3, 3]:
            valid_time = (requested_time + timedelta(days=delta_day) +
                          timedelta(hours=15))
            filename = (ice_folder_name + 'ice_conc_svalbard_' +
                        valid_time.strftime('%Y%m%d1500.nc'))
            if os.path.exists(filename):
                print('Found file:')
                print(filename)
                gdal_dataset = gdal.Open(filename)
                gdal_metadata = gdalDataset.GetMetadata()
                mg.Mapper.__init__(self, filename, gdal_dataset, gdal_metadata)
                found_dataset = True
                # Modify GeoTransform from netCDF file
                # - otherwise a shift is seen!
                self.dataset.SetGeoTransform(
                    (-1243508 - 1000, 1000, 0, -210526 - 7000, 0, -1000))
                break  # Data is found for this day

        if found_dataset is False:
            AttributeError("No local Svalbard-ice files available")
            sys.exit()
